/*
 *  subtitle editor
 *  Copyright @ 2005-2006, IDJAAD Djamel (aka Kitone)
 *
 *  Contact: kitone at free point fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author IDJAAD Djamel (kitone at free point fr)
 */

#include "Dialog.h"
#include <gtkmm/stock.h>
#include <gtkmm/comboboxtext.h>
#include <iostream>
#include <sstream>
#include <gtkmm.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "utility.h"

#include <libglademm/xml.h>

/*
 *  HIG for dialog  
 */
void setDialog(Gtk::Dialog &dialog)
{
  dialog.set_border_width(6);
  dialog.set_resizable (false);
  dialog.set_has_separator (false);
  dialog.set_skip_taskbar_hint (true);
  dialog.get_vbox ()->set_spacing (8);
}


/*
 * GtkSubtitleTimer
 */
GtkSubtitleTime::GtkSubtitleTime()
:Gtk::Table(1,8,true),
 m_labelHours(_("Hours"),0.0,0.5),
 m_labelMins(_("Mins"),0.0,0.5),
 m_labelSecs(_("Secs"),0.0,0.5),
 m_labelMSecs(_("MSecs"),0.0,0.5)
{
  set_col_spacings(2);
 
  attach(m_spinHours, 0,1,0,1);
  attach(m_labelHours, 1,2,0,1);
  
  attach(m_spinMins, 2,3,0,1);
  attach(m_labelMins, 3,4,0,1);
  
  attach(m_spinSecs, 4,5,0,1);
  attach(m_labelSecs, 5,6,0,1);
  
  attach(m_spinMSecs, 6,7,0,1);
  attach(m_labelMSecs, 7,8,0,1);
  
  m_spinHours.set_range(0,24);
  m_spinHours.set_increments(1,1);

  m_spinMins.set_range(0,60);
  m_spinMins.set_increments(1,1);

  m_spinSecs.set_range(0,60);
  m_spinSecs.set_increments(1,1);

  m_spinMSecs.set_range(0,999);
  m_spinMSecs.set_increments(1,1);

  show_all();
}

void GtkSubtitleTime::set_value(int h, int m, int s, int ms)
{
  m_spinHours.set_value(h);
  m_spinMins.set_value(m);
  m_spinSecs.set_value(s);
  m_spinMSecs.set_value(ms);
}
  
void GtkSubtitleTime::set_value(const SubtitleTime &time)
{
  m_spinHours.set_value(time.hours);
  m_spinMins.set_value(time.mins);
  m_spinSecs.set_value(time.secs);
  m_spinMSecs.set_value(time.msecs);
}

SubtitleTime GtkSubtitleTime::get_value()
{
  int h = m_spinHours.get_value_as_int();
  int m = m_spinMins.get_value_as_int();
  int s = m_spinSecs.get_value_as_int();
  int ms= m_spinMSecs.get_value_as_int();
  
  return SubtitleTime(h, m, s, ms);
}





/*
 *  DIALOG FIND
 */
DialogFind::DialogFind()
:Gtk::Dialog(_("Find")), m_table(1,4,true), 
  m_labelFind(_("Find what:"),1.0,0.5)
{
  setDialog(*this);

  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);

  get_vbox()->pack_start(m_table);
  m_table.set_col_spacings(3);

  m_table.attach(m_labelFind, 0,1,0,1);
  m_table.attach(m_entryFind, 1,4,0,1);

  m_table.show_all();
}

Glib::ustring DialogFind::getFindText()
{
  return m_entryFind.get_text();
}



/*
 *  DIALOG FIND AND REPLACE 
 */
DialogFindAndReplace::DialogFindAndReplace()
:Gtk::Dialog(_("Find And Replace")), m_table(2,4,true), 
  m_labelFind(_("Find what:"),1.0,0.5), m_labelReplace(_("Replace with:"), 1.0,0.5) 
{
  setDialog(*this);

  add_button(Gtk::Stock::CLOSE, CLOSE);
  add_button(_("Replace All"), REPLACE_ALL);
  add_button(_("Replace"), REPLACE);
  add_button(Gtk::Stock::FIND, FIND);

  get_vbox()->pack_start(m_table);
  m_table.set_col_spacings(3);

  m_table.attach(m_labelFind, 0,1,0,1);
  m_table.attach(m_entryFind, 1,4,0,1);
 
  m_table.attach(m_labelReplace, 0,1,1,2);
  m_table.attach(m_entryReplace, 1,4,1,2);
 
  m_table.show_all();
}

Glib::ustring DialogFindAndReplace::getFindText()
{
  return m_entryFind.get_text();
}

Glib::ustring DialogFindAndReplace::getReplaceText()
{
  return m_entryReplace.get_text();
}


/*
 *  DIALOG About
 */
DialogAbout::DialogAbout(Gtk::Window &win)
//:Gtk::AboutDialog("About", win, true,false)
{
  set_transient_for(win);

  // name
  set_name("Subtitle Editor");
  // version
  set_version(VERSION);
  // copyright
  set_copyright("kitone (IDJAAD djamel)");
  // comments
  set_comments("a tool for edit subtitle");

  // authors
  std::list<Glib::ustring>  authors;
  authors.push_back("kitone (IDJAAD djamel)");
  set_authors(authors);

  // translator-credits
  //std::list<Glib::ustring>  translators;
  //set_translator_credits(_("translator-credits"));
  
  // website
  set_website("http://kitone.free.fr/subtitleeditor/");
  set_website_label("http://kitone.free.fr/subtitleeditor/");
  
  // license
  Glib::ustring license=
    "This program is free software; you can redistribute it and/or modify  \n"
    "it under the terms of the GNU General Public License as published by  \n"
    "the Free Software Foundation; either version 2 of the License, or  \n"
    "(at your option) any later version.  \n\n"
    "This program is distributed in the hope that it will be useful,  \n"
    "but WITHOUT ANY WARRANTY; without even the implied warranty of  \n"
    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  \n"
    "GNU General Public License for more details.  \n\n"
    "You should have received a copy of the GNU General Public License  \n"
    "along with this program; if not, write to the Free Software  \n"
    "Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA  \n\n"
    "See gpl.txt for more information regarding the GNU General Public License. \n";
  set_license(license);

	try
	{
		Glib::ustring filename = get_share_dir("subtitleeditor.png");
		set_logo(Gdk::Pixbuf::create_from_file(filename));
	}
	catch(Glib::Error &ex)
	{
		std::cerr << ex.what() << std::endl;
	}
}


/*
 *  DIALOG OPEN SRT
 */
DialogOpenSubtitle::DialogOpenSubtitle()
:Gtk::FileChooserDialog(_("Open Subtitle"))
{
  Gtk::FileFilter m_filter;
  m_filter.set_name("subtitles");
  m_filter.add_pattern("*.srt");
  m_filter.add_pattern("*.SRT");
  m_filter.add_pattern("*.ssa");
  m_filter.add_pattern("*.SSA");
  add_filter(m_filter);

  Gtk::FileFilter m_filterAll;
  m_filterAll.set_name("ALL");
  m_filterAll.add_pattern("*.*");
  add_filter(m_filterAll);

  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);

}

/*
 *  DIALOG SAVE SRT
 */
DialogSaveSubtitle::DialogSaveSubtitle()
:Gtk::FileChooserDialog(_("Save Subtitle"), Gtk::FILE_CHOOSER_ACTION_SAVE)
{
  Gtk::FileFilter m_filterSRT;
  m_filterSRT.set_name("srt");
  m_filterSRT.add_pattern("*.srt");
  add_filter(m_filterSRT);

  Gtk::FileFilter m_filterSSA;
  m_filterSSA.set_name("ssa");
  m_filterSSA.add_pattern("*.ssa");
  add_filter(m_filterSSA);
/*
	Gtk::FileFilter m_filterAll;
  m_filterAll.set_name("ALL");
  m_filterAll.add_pattern("*.*");
  add_filter(m_filterAll);
*/
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);

  /*
  Gtk::ComboBoxText *comboEncoding;

  comboEncoding = manage(new Gtk::ComboBoxText);


  unsigned int size = sizeof(character_encodings) / sizeof(character_encodings[0]);
  for(unsigned int i=0; i<size; ++i)
  {
    comboEncoding->append_text(character_encodings[i]);
  }
  comboEncoding->set_active_text("UTF-8");

  get_vbox()->pack_start(*comboEncoding, false,false);
  */
  get_vbox()->show_all();
}

/*
 *  DIALOG IMPORT TXT
 */
DialogImportText::DialogImportText()
:Gtk::FileChooserDialog(_("Import Text"))
{
  /*
  Gtk::FileFilter m_filter;
  m_filter.set_name("subtitle");
  m_filter.add_pattern("*.srt");
  add_filter(m_filter);

  Gtk::FileFilter m_filterAll;
  m_filterAll.set_name("ALL");
  m_filterAll.add_pattern("*.*");
  add_filter(m_filterAll);
  */
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
}


/*
 *  DIALOG Export TXT
 */
DialogExportText::DialogExportText()
:Gtk::FileChooserDialog(_("Export Text"), Gtk::FILE_CHOOSER_ACTION_SAVE)
{
  /*
  Gtk::FileFilter m_filter;
  m_filter.set_name("text");
  m_filter.add_pattern("*.txt");
  add_filter(m_filter);

  Gtk::FileFilter m_filterAll;
  m_filterAll.set_name("ALL");
  m_filterAll.add_pattern("*.*");
  add_filter(m_filterAll);
  */
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
}

/*
 *  DIALOG OPEN MOVIE
 */
DialogOpenMovie::DialogOpenMovie()
:Gtk::FileChooserDialog(_("Open Movie"))
{
  Gtk::FileFilter m_filter;
  m_filter.set_name("movies");
  m_filter.add_pattern("*.avi");
  m_filter.add_pattern("*.mpg");
  m_filter.add_pattern("*.wma");
  m_filter.add_pattern("*.mkv");
  m_filter.add_pattern("*.mpeg");
  //...
  add_filter(m_filter);

  Gtk::FileFilter m_filterAll;
  m_filterAll.set_name("ALL");
  m_filterAll.add_pattern("*.*");
  add_filter(m_filterAll);
  
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
}


/*
 * DIALOG Scale Subttile
 */
DialogScaleSubtitle::DialogScaleSubtitle()
:Gtk::Dialog(_("Scale")), m_table(4,4,true), 
  m_labelSource1(_("Source1 :"), 1.0,0.5), m_labelDest1("Dest1 :", 1.0, 0.5),
  m_labelSource2(_("Source2 :"), 1.0,0.5), m_labelDest2("Dest2 :", 1.0, 0.5)
{
  setDialog(*this);

  m_table.set_row_spacing(1,6);
  m_table.set_col_spacing(0,6);

  get_vbox()->pack_start(m_table);

  m_table.attach(m_labelSource1, 0,1,0,1);
  m_table.attach(m_subtitleTimeSource1, 1,4,0,1);
  m_table.attach(m_labelDest1, 0,1,1,2);
  m_table.attach(m_subtitleTimeDest1, 1,4,1,2);

  m_table.attach(m_labelSource2, 0,1,2,3);
  m_table.attach(m_subtitleTimeSource2, 1,4,2,3);
  m_table.attach(m_labelDest2, 0,1,3,4);
  m_table.attach(m_subtitleTimeDest2, 1,4,3,4);
 
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);

  m_table.show_all();
}

/*
 *	DIALOG Scale Subtitle (2)
 */
DialogScaleSubtitle2::DialogScaleSubtitle2(Glib::RefPtr<SubtitleModel> model)
:Gtk::Dialog(_("Scale")), m_model(model), 
	m_table(2,4,false), 
	m_labelSource1(_("Start:"), 1.0, 0.5), m_labelDest1(_("Destination:"), 1.0, 0.5),
	m_spinSource1(1,0), 
	m_labelSource2(_("End:"), 1.0, 0.5), m_labelDest2(_("Destination:"), 1.0, 0.5),
	m_spinSource2(1,0)
{
	setDialog(*this);

  //m_table.set_row_spacing(1,6);
  m_table.set_col_spacing(0,6);
  m_table.set_col_spacing(1,12);
  m_table.set_col_spacing(2,6);

  m_spinSource1.set_increments(1,1);
  m_spinSource2.set_increments(1,1);

	m_entrySource1.set_max_length(12);
	m_entrySource1.set_width_chars(12);
	m_entrySource2.set_max_length(12);
	m_entrySource2.set_width_chars(12);
	
	get_vbox()->pack_start(m_table);

	m_table.attach(m_labelSource1, 0,1,0,1);
	m_table.attach(m_spinSource1, 1,2,0,1);
	m_table.attach(m_labelDest1, 2,3,0,1);
	m_table.attach(m_entrySource1, 3,4,0,1);

	m_table.attach(m_labelSource2, 0,1,1,2);
	m_table.attach(m_spinSource2, 1,2,1,2);
	m_table.attach(m_labelDest2, 2,3,1,2);
	m_table.attach(m_entrySource2, 3,4,1,2);
	

	m_entrySource1.set_text("00:00:00,000");
	m_entrySource2.set_text("00:00:00,000");

  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	
	m_spinSource1.signal_value_changed().connect(
			sigc::mem_fun(*this, &DialogScaleSubtitle2::on_spin1_changed));
	
	m_spinSource2.signal_value_changed().connect(
			sigc::mem_fun(*this, &DialogScaleSubtitle2::on_spin2_changed));

	m_entrySource1.signal_activate().connect(
			sigc::mem_fun(*this, &DialogScaleSubtitle2::on_entry1_changed));
	
	m_entrySource2.signal_activate().connect(
			sigc::mem_fun(*this, &DialogScaleSubtitle2::on_entry2_changed));


	int min = 1;
	int max = model->getSize();
	
	m_spinSource1.set_range(min, max);
	m_spinSource2.set_range(min, max);
	
	m_spinSource1.set_value(min);
	m_spinSource2.set_value(max);

	m_table.show_all();
}

void DialogScaleSubtitle2::on_spin1_changed()
{
	unsigned int i = (unsigned int)m_spinSource1.get_value();

	Gtk::TreeIter iter = m_model->find(i);
	if(iter)
	{
		SubtitleColumnRecorder column;
		m_entrySource1.set_text((*iter)[column.start]);
	}
}

void DialogScaleSubtitle2::on_spin2_changed()
{
	unsigned int i = (unsigned int)m_spinSource2.get_value();

	Gtk::TreeIter iter = m_model->find(i);
	if(iter)
	{
		SubtitleColumnRecorder column;
		m_entrySource2.set_text((*iter)[column.start]);
	}
}

void DialogScaleSubtitle2::on_entry1_changed()
{
	Glib::ustring text = m_entrySource1.get_text();
	if(check_time(text) == false)
		m_entrySource1.set_text("00:00:00,000");
}

void DialogScaleSubtitle2::on_entry2_changed()
{
	Glib::ustring text = m_entrySource2.get_text();
	if(check_time(text) == false)
		m_entrySource2.set_text("00:00:00,000");
}

/*
 * DIALOG Edit Subtitle
 */
DialogEditSubtitle::DialogEditSubtitle()
:Gtk::Dialog(_("Edit Subtitle")), 
  m_table(3,3,true),
  m_labelText(_("text:"), 1.0,0.5),
  m_labelStart(_("start:"), 1.0,0.5),
  m_labelEnd(_("end:"), 1.0,0.5)
{
  setDialog(*this);

  get_vbox()->pack_start(m_table);

  m_table.set_col_spacing(0,6);

  m_table.attach(m_labelStart, 0,1,0,1);
  m_table.attach(m_timerStart, 1,4,0,1);

  m_table.attach(m_labelEnd, 0,1,1,2);
  m_table.attach(m_timerEnd, 1,4,1,2);
  
  m_table.attach(m_labelText, 0,1,2,3);
  m_table.attach(m_entryText, 1,4,2,3);

  m_table.set_row_spacing(1,6);
 
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);

  m_table.show_all();
}
/*
void DialogEditSubtitle::set_value(Gtk::TreeIter &iter)
{

  m_subtitleTmp = sub;

  m_timerStart.set_value(sub.start);
  m_timerEnd.set_value(sub.end);

  m_entryText.set_text(sub.text);
}

Subtitle DialogEditSubtitle::get_value()
{
  Subtitle res;
  res.num   = m_subtitleTmp.num;
  res.start = m_timerStart.get_value();
  res.end   = m_timerEnd.get_value();
  res.text  = m_entryText.get_text();
  return res;
}
*/

/*
 *  DIALOG Move Subtitle
 */
DialogMoveSubtitle::DialogMoveSubtitle()
:Gtk::Dialog(_("Move Subtitle")),
  m_table(4,2,false), 
  //m_labelDest(_("How much time should be added ?"), 0.0,0.5),
  m_labelDest(_("Enter time that you want:"), 0.0,0.5),
  m_labelOption(_("Which subtitle ?"), 0.0,0.5),
  m_radioAll(_("All subtitles")),
  m_radioCurrent(_("Current line"))
{
  setDialog(*this);

  get_vbox()->pack_start(m_table);

  m_table.set_row_spacings(5);
  
  m_table.attach(m_labelDest, 0,2,0,1);
  m_table.attach(m_timer, 0,2,1,2, 
      Gtk::FILL | Gtk::EXPAND,
      Gtk::FILL | Gtk::EXPAND,
      12,0);
  /*
  Gtk::RadioButtonGroup group = m_radioAll.get_group();
  m_radioCurrent.set_group(group);

  m_table.attach(m_labelOption, 0,2,2,3);
  m_table.attach(m_radioAll, 0,2,3,4, 
      Gtk::FILL | Gtk::EXPAND,
      Gtk::FILL | Gtk::EXPAND,
      12,0);
  m_table.attach(m_radioCurrent, 0,2,4,5, 
      Gtk::FILL | Gtk::EXPAND,
      Gtk::FILL | Gtk::EXPAND,
      12,0);
  
  m_table.set_row_spacing(1,12);
  */
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);

  m_table.show_all();
}

void DialogMoveSubtitle::set_value(const SubtitleTime &start)
{
  m_timer.set_value(start);
}

SubtitleTime DialogMoveSubtitle::get_value()
{
  return m_timer.get_value();
}

/*
 * DIALOG Change FPS
 */
DialogChangeFPS::DialogChangeFPS()
:Gtk::Dialog(_("Change FPS")),
  m_table(2,2,true),
  m_labelSrc(_("Src FPS:"), 1.0,0.5),
  m_labelDest(_("Dest FPS:"), 1.0,0.5),
  m_spinSrc(1,3), m_spinDest(1,3)
{
  setDialog(*this);

  // default value
  m_spinSrc.set_increments(1,1);
  m_spinSrc.set_range(0.0,100.0);
  m_spinSrc.set_value(25);

  m_spinDest.set_increments(1,1);
  m_spinDest.set_range(0.0,100.0);
  m_spinDest.set_value(23.970);
 
  get_vbox()->pack_start(m_table);

  m_table.set_col_spacing(0,6);

  m_table.attach(m_labelSrc, 0,1,0,1);
  m_table.attach(m_spinSrc, 1,2,0,1);

  m_table.attach(m_labelDest, 0,1,1,2);
  m_table.attach(m_spinDest, 1,2,1,2);
  
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);

  m_table.show_all();
}

double DialogChangeFPS::get_source()
{
  return m_spinSrc.get_value();
}

double DialogChangeFPS::get_dest()
{
  return m_spinDest.get_value();
}


DialogSetAllEndTime::DialogSetAllEndTime()
:Gtk::Dialog(_("Set All End Time")), 
  m_table(5,2, false),
  m_labelRadio(_("<b>Option:</b>"),0,.5),
  m_radioAdd(_("Add Time")), 
  m_radioSet(_("Set Time")),
  m_label(_("<b>Please enter time for added or initialized:</b>"),0.0,0.5)
{
  setDialog(*this);

  
  m_label.set_use_markup(true);
  m_labelRadio.set_use_markup(true);
  
  m_table.set_row_spacings(5);
  m_table.set_row_spacing(1,12);

  Gtk::RadioButtonGroup group = m_radioAdd.get_group();

  m_radioSet.set_group(group);
  
  
  get_vbox()->add(m_table);

  m_table.attach(m_label, 0,2,0,1);
  m_table.attach(m_time, 0,2,1,2,
      Gtk::FILL | Gtk::EXPAND,
      Gtk::FILL | Gtk::EXPAND,
      12,0);


  m_table.attach(m_labelRadio, 0,2,2,3);
  m_table.attach(m_radioAdd, 0,1,3,4, 
      Gtk::FILL | Gtk::EXPAND,
      Gtk::FILL | Gtk::EXPAND,
      12,0);
  m_table.attach(m_radioSet, 0,1,4,5,
      Gtk::FILL | Gtk::EXPAND,
      Gtk::FILL | Gtk::EXPAND,
      12,0);


  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);

  m_table.show_all();
}

DialogSetAllEndTime::Type DialogSetAllEndTime::get_type()
{
  if(m_radioAdd.get_active())
    return ADD;
  else 
    return SET;
}

SubtitleTime DialogSetAllEndTime::get_time()
{
  return m_time.get_value();
}



/*
 *
 */
DialogStyleEditor::DialogStyleEditor(Document* document)
:Gtk::Dialog(_("Subtitle Editor - Style Editor")), ref_document(document), ref_styleModel(document->m_styleModel)
{
	setDialog(*this);

	Glib::RefPtr<Gnome::Glade::Xml> refXml;

	try
	{
		Glib::ustring filename = get_share_dir("subtitleeditor.glade");
		refXml = Gnome::Glade::Xml::create(filename, "hbox1");
	}
	catch(const Glib::Error &ex)
	{
		std::cerr << ex.what() << std::endl;
		return;
	}

	Gtk::Widget *hbox1 = NULL;

	refXml->get_widget("hbox1", hbox1);
	
	get_vbox()->pack_start(*hbox1);


	//
	//
	entryName = NULL;
	buttonFont = NULL;
	buttonBold = NULL;
	buttonItalic = NULL;
	buttonUnderline = NULL;
	buttonStrikeout = NULL;

	buttonPrimaryColor = NULL;
	buttonSecondaryColor = NULL;
	buttonOutlineColor = NULL;
	buttonShadowColor = NULL;
	
	spinMarginL = NULL;
	spinMarginR = NULL;
	spinMarginV = NULL;

	spinAngle = NULL;
	spinScaleX = NULL;
	spinScaleY = NULL;
	spinSpacing = NULL;

	radioOutline = NULL;
	spinOutline = NULL;
	spinShadow = NULL;
	radioOpaqueBox = NULL;
	//
	//

	refXml->get_widget("vboxEdit", vbox2);

	refXml->get_widget("buttonNewStyle", buttonNewStyle);
	refXml->get_widget("buttonDeleteStyle", buttonDeleteStyle);
	refXml->get_widget("buttonCopyStyle", buttonCopyStyle);
	refXml->get_widget("buttonManageStyles", buttonManageStyles);
	refXml->get_widget("treeviewStyle", treeviewStyle);

	refXml->get_widget("entryName", entryName);
	refXml->get_widget("buttonFont", buttonFont);
	
	refXml->get_widget("buttonBold", buttonBold);
	refXml->get_widget("buttonItalic", buttonItalic);
	refXml->get_widget("buttonUnderline", buttonUnderline);
	refXml->get_widget("buttonStrikeout", buttonStrikeout);

	refXml->get_widget("buttonPrimaryColor", buttonPrimaryColor);
	refXml->get_widget("buttonSecondaryColor", buttonSecondaryColor);
	refXml->get_widget("buttonOutlineColor", buttonOutlineColor);
	refXml->get_widget("buttonShadowColor", buttonShadowColor);
	
	refXml->get_widget("spinMarginL", spinMarginL);
	refXml->get_widget("spinMarginR", spinMarginR);
	refXml->get_widget("spinMarginV", spinMarginV);

	refXml->get_widget("spinAngle", spinAngle);
	refXml->get_widget("spinScaleX", spinScaleX);
	refXml->get_widget("spinScaleY", spinScaleY);
	refXml->get_widget("spinSpacing", spinSpacing);

	refXml->get_widget("radioOutline", radioOutline);
	refXml->get_widget("spinOutline", spinOutline);
	refXml->get_widget("spinShadow", spinShadow);
	refXml->get_widget("radioOpaqueBox", radioOpaqueBox);

	refXml->get_widget("buttonAlignment1", buttonAlignment[0]);
	refXml->get_widget("buttonAlignment2", buttonAlignment[1]);
	refXml->get_widget("buttonAlignment3", buttonAlignment[2]);
	refXml->get_widget("buttonAlignment4", buttonAlignment[3]);
	refXml->get_widget("buttonAlignment5", buttonAlignment[4]);
	refXml->get_widget("buttonAlignment6", buttonAlignment[5]);
	refXml->get_widget("buttonAlignment7", buttonAlignment[6]);
	refXml->get_widget("buttonAlignment8", buttonAlignment[7]);
	refXml->get_widget("buttonAlignment9", buttonAlignment[8]);
	//


	//
	{
		treeviewStyle->set_model(ref_styleModel);

		Gtk::TreeViewColumn* _column = NULL;
		Gtk::CellRendererText* _renderer = NULL;

		_column = manage(new Gtk::TreeViewColumn(_("Styles")));
		_renderer = manage(new Gtk::CellRendererText());

		_column->pack_start(*_renderer, false);
		_column->add_attribute(_renderer->property_text(), column.name);

		treeviewStyle->append_column(*_column);

		treeviewStyle->get_selection()->signal_changed().connect(
				sigc::mem_fun(*this, &DialogStyleEditor::on_select_style_changed));
	}
	
	for(unsigned int i=0; i<9; ++i)
	{
		buttonAlignment[i]->signal_toggled().connect(
				sigc::bind<unsigned int>(sigc::mem_fun(*this, &DialogStyleEditor::on_button_alignment_changed), i+1));
	}
	// signaux
#define CONNECT(widget,signal,callback) widget->signal().connect(sigc::mem_fun(*this, &DialogStyleEditor::callback));

	CONNECT(buttonNewStyle, signal_clicked, on_new_style);
	CONNECT(buttonDeleteStyle, signal_clicked, on_delete_style);
	CONNECT(buttonCopyStyle, signal_clicked, on_copy_style);
	CONNECT(buttonManageStyles, signal_clicked, on_manage_styles);

	CONNECT(entryName, signal_activate, on_entry_name_changed);

	CONNECT(buttonFont, signal_font_set, on_button_font_changed);

	CONNECT(buttonBold, signal_toggled, on_button_bold_toggled);
	CONNECT(buttonItalic, signal_toggled, on_button_italic_toggled);
	CONNECT(buttonUnderline, signal_toggled, on_button_underline_toggled);
	CONNECT(buttonStrikeout, signal_toggled, on_button_strikeout_toggled);

	CONNECT(buttonPrimaryColor, signal_color_set, on_color_primary_changed);
	CONNECT(buttonSecondaryColor, signal_color_set, on_color_secondary_changed);
	CONNECT(buttonOutlineColor, signal_color_set, on_color_outline_changed);
	CONNECT(buttonShadowColor, signal_color_set, on_color_shadow_changed);
	
	CONNECT(spinMarginL, signal_value_changed, on_spin_margin_l_changed);
	CONNECT(spinMarginR, signal_value_changed, on_spin_margin_r_changed);
	CONNECT(spinMarginV, signal_value_changed, on_spin_margin_v_changed);

	CONNECT(spinAngle, signal_value_changed, on_spin_angle_changed);
	CONNECT(spinScaleX, signal_value_changed, on_spin_scale_x_changed);
	CONNECT(spinScaleY, signal_value_changed, on_spin_scale_y_changed);
	CONNECT(spinSpacing, signal_value_changed, on_spin_spacing_changed);

	CONNECT(radioOutline, signal_toggled, on_radio_outline_changed);
	CONNECT(spinOutline, signal_value_changed, on_spin_outline_changed);
	CONNECT(spinShadow, signal_value_changed, on_spin_shadow_changed);
	CONNECT(radioOpaqueBox, signal_toggled, on_radio_opaque_box_changed);
	
#undef CONNECT

	on_select_style_changed();

	show_all();
	run();
}

/*
 *	y a t'il un style select pour edition
 */
bool DialogStyleEditor::has_select()
{
	return iter;
}

void DialogStyleEditor::on_select_style_changed()
{
	iter = treeviewStyle->get_selection()->get_selected();
	
	if(iter)
	{
		vbox2->set_sensitive(true);
		on_init_style();
	}
	else
	{
		vbox2->set_sensitive(false);
	}
}

/*
 *	init l'interface avec le style select
 */
void DialogStyleEditor::on_init_style()
{
	g_return_if_fail(has_select());

	entryName->set_text( (*iter)[column.name] );

	//font
	std::ostringstream oss_font_size;

	oss_font_size << (*iter)[column.font_size];
	
	Glib::ustring font_name_size = (*iter)[column.font_name] + " " + oss_font_size.str();
	buttonFont->set_font_name( font_name_size);


	buttonBold->set_active( (*iter)[column.bold] );
	buttonItalic->set_active( (*iter)[column.italic] );
	buttonUnderline->set_active( (*iter)[column.underline] );
	buttonStrikeout->set_active( (*iter)[column.strikeout] );

	// color
	Gdk::Color color;

	uint2color( (*iter)[column.primary_colour] , color);
	buttonPrimaryColor->set_color(color);

	uint2color( (*iter)[column.secondary_colour] , color);
	buttonSecondaryColor->set_color(color);
	
	uint2color( (*iter)[column.outline_colour] , color);
	buttonOutlineColor->set_color(color);
	
	uint2color( (*iter)[column.shadow_colour] , color);
	buttonShadowColor->set_color(color);


	// margin
	spinMarginL->set_value( (*iter)[column.margin_l] );
	spinMarginR->set_value( (*iter)[column.margin_r] );
	spinMarginV->set_value( (*iter)[column.margin_v] );

	// font angle,scale x, scale y, spacing
	spinAngle->set_value( (*iter)[column.angle] );
	spinScaleX->set_value( (*iter)[column.scale_x] );
	spinScaleY->set_value( (*iter)[column.scale_y] );
	spinSpacing->set_value( (*iter)[column.spacing] );

	if((*iter)[column.border_style] == 1)
		radioOutline->set_active(true);
	else
		radioOpaqueBox->set_active(true);

	spinOutline->set_value((*iter)[column.outline]);
	spinShadow->set_value((*iter)[column.shadow]);

	buttonAlignment[(*iter)[column.alignment] -1 ]->set_active(true);
}

void DialogStyleEditor::on_entry_name_changed()
{
	g_return_if_fail(has_select());
	
	Glib::ustring text = entryName->get_text();

	(*iter)[column.name] = text;
}

void DialogStyleEditor::on_button_font_changed()
{
	g_return_if_fail(has_select());

	Glib::ustring font_name = buttonFont->get_font_name();

	std::cout << font_name << std::endl;
}
	
void DialogStyleEditor::on_button_bold_toggled()
{
	g_return_if_fail(has_select());

	bool state = buttonBold->get_active();

	(*iter)[column.bold] = state;
}

void DialogStyleEditor::on_button_italic_toggled()
{
	g_return_if_fail(has_select());

	bool state = buttonItalic->get_active();

	(*iter)[column.italic] = state;
}

void DialogStyleEditor::on_button_underline_toggled()
{
	g_return_if_fail(has_select());

	bool state = buttonUnderline->get_active();

	(*iter)[column.underline] = state;
}

void DialogStyleEditor::on_button_strikeout_toggled()
{
	g_return_if_fail(has_select());

	bool state = buttonStrikeout->get_active();

	(*iter)[column.strikeout] = state;
}

void DialogStyleEditor::on_color_primary_changed()
{
	g_return_if_fail(has_select());

	Gdk::Color color = buttonPrimaryColor->get_color();

	unsigned int value = 0;
	color2uint(color, value);

	(*iter)[column.primary_colour] = value;
}

void DialogStyleEditor::on_color_secondary_changed()
{
	g_return_if_fail(has_select());

	Gdk::Color color = buttonSecondaryColor->get_color();

	unsigned int value = 0;
	color2uint(color, value);

	(*iter)[column.secondary_colour] = value;
}

void DialogStyleEditor::on_color_outline_changed()
{
	g_return_if_fail(has_select());

	Gdk::Color color = buttonOutlineColor->get_color();

	unsigned int value = 0;
	color2uint(color, value);

	(*iter)[column.outline_colour] = value;
}

void DialogStyleEditor::on_color_shadow_changed()
{
	g_return_if_fail(has_select());

	Gdk::Color color = buttonShadowColor->get_color();
	unsigned int value = 0;
	color2uint(color, value);

	(*iter)[column.shadow_colour] = value;
}

void DialogStyleEditor::on_spin_margin_l_changed()
{
	g_return_if_fail(has_select());

	double value = spinMarginL->get_value();

	(*iter)[column.margin_l] = (unsigned int) value;
}

void DialogStyleEditor::on_spin_margin_r_changed()
{
	g_return_if_fail(has_select());

	double value = spinMarginR->get_value();

	(*iter)[column.margin_r] = (unsigned int) value;
}

void DialogStyleEditor::on_spin_margin_v_changed()
{
	g_return_if_fail(has_select());

	double value = spinMarginV->get_value();

	(*iter)[column.margin_v] = (unsigned int) value;
}

void DialogStyleEditor::on_spin_angle_changed()
{
	g_return_if_fail(has_select());

	double value = spinAngle->get_value();

	(*iter)[column.angle] = (unsigned int) value;
}

void DialogStyleEditor::on_spin_scale_x_changed()
{
	g_return_if_fail(has_select());

	double value = spinScaleX->get_value();

	(*iter)[column.scale_x] = (unsigned int) value;
}

void DialogStyleEditor::on_spin_scale_y_changed()
{
	g_return_if_fail(has_select());

	double value = spinScaleY->get_value();

	(*iter)[column.scale_y] = (unsigned int) value;
}

void DialogStyleEditor::on_spin_spacing_changed()
{
	g_return_if_fail(has_select());

	double value = spinSpacing->get_value();

	(*iter)[column.spacing] = (unsigned int) value;
}



void DialogStyleEditor::on_radio_outline_changed()
{
	g_return_if_fail(has_select());

	if(radioOutline->get_active())
		(*iter)[column.border_style] = 1;
}

void DialogStyleEditor::on_spin_outline_changed()
{
	g_return_if_fail(has_select());

	double value = spinOutline->get_value();

	(*iter)[column.outline] = (unsigned int) value;
}

void DialogStyleEditor::on_spin_shadow_changed()
{
	g_return_if_fail(has_select());

	double value = spinShadow->get_value();

	(*iter)[column.shadow] = (unsigned int) value;
}

void DialogStyleEditor::on_radio_opaque_box_changed()
{
	g_return_if_fail(has_select());

	if(radioOpaqueBox->get_active())
		(*iter)[column.border_style] = 3;
}

	
void DialogStyleEditor::on_button_alignment_changed(unsigned int num)
{
	g_return_if_fail(has_select());

	if(buttonAlignment[num-1]->get_active())
		(*iter)[column.alignment] = num;
}

void DialogStyleEditor::on_new_style()
{
	Gtk::TreeIter it = ref_styleModel->append();
	(*it)[column.name] = "New Style";

	treeviewStyle->get_selection()->select(it);
}

void DialogStyleEditor::on_delete_style()
{
	if(has_select() == false)
		return;
	
	iter = ref_styleModel->erase(iter);
		
	if(iter)
		treeviewStyle->get_selection()->select(iter);

#warning "check SubtitleModel for remove style erase"

}

void DialogStyleEditor::on_copy_style()
{
	if(has_select() == false)
		return;

	Gtk::TreeIter src = iter;

	Gtk::TreeIter dst = ref_styleModel->copy(src);

	(*dst)[column.name] = (*dst)[column.name] + "-copy";

	treeviewStyle->get_selection()->select(dst);
}

void DialogStyleEditor::on_manage_styles()
{
	std::cerr << "DialogStyleEditor::on_manage_styles()" << std::endl;
}




DialogScriptProperties::DialogScriptProperties(Document *doc)
:Gtk::Dialog(_("Subtitle Editor - Script Properties"))
{
	//set_default_size(500,500);

	setDialog(*this);

	Glib::RefPtr<Gnome::Glade::Xml> refXml;

	try
	{
		Glib::ustring filename = get_share_dir("subtitleeditor.glade");
		refXml = Gnome::Glade::Xml::create(filename, "vboxScriptProperties");
	}
	catch(const Glib::Error &ex)
	{
		std::cerr << ex.what() << std::endl;
		return;
	}

	Gtk::VBox *vboxScriptProperties = NULL;

	refXml->get_widget("vboxScriptProperties", vboxScriptProperties);

	get_vbox()->pack_start(*vboxScriptProperties);


	refXml->get_widget("entryScriptType", entryScriptType);
	refXml->get_widget("entryPointTime", entryPointTime);

	refXml->get_widget("entryTitle", entryTitle);
	refXml->get_widget("entryOriginalScript", entryOriginalScript);
	refXml->get_widget("entryOriginalTranslation", entryOriginalTranslation);
	refXml->get_widget("entryOriginalEditing", entryOriginalEditing);
	refXml->get_widget("entryOriginalTiming", entryOriginalTiming);
	refXml->get_widget("entrySyncPoint", entrySyncPoint);
	refXml->get_widget("entryScriptUpdatedBy", entryScriptUpdatedBy);
	refXml->get_widget("entryUpdateDetails", entryUpdateDetails);

	refXml->get_widget("entryPlayResX", entryPlayResX);
	refXml->get_widget("entryPlayResY", entryPlayResY);
	refXml->get_widget("entryPlayDepth", entryPlayDepth);

	refXml->get_widget("comboCollisions", comboCollisions);
	refXml->get_widget("entryWarpStyle", entryWarpStyle);

	refXml->get_widget("spinTimer", spinTimer);

	show_all();

	
	init(&doc->m_scriptInfo);


#define CONNECT(widget,signal,callback) widget->signal().connect(sigc::mem_fun(*this, &DialogScriptProperties::callback));

	CONNECT(entryScriptType, signal_changed, on_entry_script_type_changed);
	CONNECT(entryPointTime, signal_changed, on_entry_point_time_changed);

	CONNECT(entryTitle, signal_changed, on_entry_title_changed);
	CONNECT(entryOriginalScript, signal_changed, on_entry_original_script_changed);
	CONNECT(entryOriginalTranslation, signal_changed, on_entry_original_translation_changed);
	CONNECT(entryOriginalEditing, signal_changed, on_entry_original_editing_changed);
	CONNECT(entryOriginalTiming, signal_changed, on_entry_original_timing_changed);
	CONNECT(entrySyncPoint, signal_changed, on_entry_sync_point_changed);
	CONNECT(entryScriptUpdatedBy, signal_changed, on_entry_script_updated_by_changed);
	CONNECT(entryUpdateDetails, signal_changed, on_entry_update_details_changed);
	
	CONNECT(entryPlayResX, signal_changed, on_entry_play_res_x_changed);
	CONNECT(entryPlayResY, signal_changed, on_entry_play_res_y_changed);
	CONNECT(entryPlayDepth, signal_changed, on_entry_play_depth_changed);

	CONNECT(comboCollisions, signal_changed, on_combo_collisions_changed);
	CONNECT(entryWarpStyle, signal_changed, on_entry_warp_style_changed);

	CONNECT(spinTimer, signal_value_changed, on_spin_timer_changed);

#undef CONNECT


	run();
}

void DialogScriptProperties::init(ScriptInfo *info)
{
	g_return_if_fail(info);

	scriptInfo = info;
	
	entryScriptType->set_text(info->ScriptType);
	//entryPointTime->set_text(info->);
	
	entryTitle->set_text(info->Title);
	entryOriginalScript->set_text(info->OriginalScript);
	entryOriginalTranslation->set_text(info->OriginalTranslation);
	entryOriginalEditing->set_text(info->OriginalEditing);
	entryOriginalTiming->set_text(info->OriginalTiming);
	entrySyncPoint->set_text(info->SynchPoint);
	entryScriptUpdatedBy->set_text(info->ScriptUpdatedBy);
	entryUpdateDetails->set_text(info->UpdateDetails);
	
	entryPlayResX->set_text(info->PlayResX);
	entryPlayResY->set_text(info->PlayResY);
	entryPlayDepth->set_text(info->PlayDepth);
	
	if(info->Collisions == "Normal")
		comboCollisions->set_active(0);
	else if(info->Collisions == "Reverse")
		comboCollisions->set_active(1);
	else
	{
		comboCollisions->set_active(0);
		info->Collisions = "Normal";
	}

	//entryWarpStyle->set_text(info->WarpStyle);

	double timer = 100.0;
	//std::string str_timer = info->Timer;
	if(from_string(info->Timer, timer))
		spinTimer->set_value(timer);
}

void DialogScriptProperties::on_entry_script_type_changed()
{
	scriptInfo->ScriptType = entryScriptType->get_text();
}

void DialogScriptProperties::on_entry_point_time_changed()
{
//	scriptInfo->PointT = entryPointTime->get_text();
}

	
void DialogScriptProperties::on_entry_title_changed()
{
	scriptInfo->Title = entryTitle->get_text();
}

void DialogScriptProperties::on_entry_original_script_changed()
{
	scriptInfo->OriginalScript = entryOriginalScript->get_text();
}

void DialogScriptProperties::on_entry_original_translation_changed()
{
	scriptInfo->OriginalTranslation = entryOriginalTranslation->get_text();
}

void DialogScriptProperties::on_entry_original_editing_changed()
{
	scriptInfo->OriginalEditing = entryOriginalEditing->get_text();
}

void DialogScriptProperties::on_entry_original_timing_changed()
{
	scriptInfo->OriginalTiming = entryOriginalTiming->get_text();
}

void DialogScriptProperties::on_entry_sync_point_changed()
{
	scriptInfo->SynchPoint = entrySyncPoint->get_text();
}

void DialogScriptProperties::on_entry_script_updated_by_changed()
{
	scriptInfo->ScriptUpdatedBy = entryScriptUpdatedBy->get_text();
}

void DialogScriptProperties::on_entry_update_details_changed()
{
	scriptInfo->UpdateDetails = entryUpdateDetails->get_text();
}


void DialogScriptProperties::on_entry_play_res_x_changed()
{
	scriptInfo->PlayResX = entryPlayResX->get_text();
}

void DialogScriptProperties::on_entry_play_res_y_changed()
{
	scriptInfo->PlayResY = entryPlayResY->get_text();
}

void DialogScriptProperties::on_entry_play_depth_changed()
{
	scriptInfo->PlayDepth = entryPlayDepth->get_text();
}

	
void DialogScriptProperties::on_combo_collisions_changed()
{
	if(comboCollisions->get_active() == 0)
		scriptInfo->Collisions = "Normal";
	else
		scriptInfo->Collisions = "Reverse";
}

void DialogScriptProperties::on_entry_warp_style_changed()
{
//	scriptInfo->WarpStyle = entryWarpStyle->get_text();
}

void DialogScriptProperties::on_spin_timer_changed()
{
	double value = spinTimer->get_value();

	scriptInfo->Timer = to_string(value);
}
