/*
 *  subtitle editor
 *	http://kitone.free.fr/subtitleeditor/
 *
 *  Copyright @ 2005-2006, IDJAAD Djamel (aka Kitone)
 *
 *  Contact: kitone at free point fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author IDJAAD Djamel (kitone at free point fr)
 */


#include "Document.h"
#include "SubtitleSSA.h"
#include "SubtitleSRT.h"


/*
 *
 */
Document::Document()
{
	m_subtitleModel = Glib::RefPtr<SubtitleModel>(new SubtitleModel);

	m_styleModel = Glib::RefPtr<StyleModel>(new StyleModel);

	//m_nameModel = Glib::RefPtr<NameModel>(new NameModel);
}


/*
 *
 */
Document::~Document()
{
}

/*
 *	utilise simplement l'extention pour determiner quel type de fichier
 *	"srt" ou "ssa"
 *	en cas d'erreur, retourne ssa
 */
Glib::ustring Document::getFormat()
{
	if(filenameDocument.find(".ssa") != Glib::ustring::npos)
		return "ssa";
	else if(filenameDocument.find(".srt") != Glib::ustring::npos)
		return "srt";

	return "ssa";
}

	
/*
 *
 */
void Document::clear()
{
	m_scriptInfo = ScriptInfo();
	
	filenameDocument.clear();
	filenameMovie.clear();
	
	m_subtitleModel->clear();
	m_styleModel->clear();
	//m_nameModel->clear();
}

/*
 *
 */
bool Document::open(const Glib::ustring &filename)
{
	clear();
	
	if(filename.find(".ssa") != Glib::ustring::npos)
	{
		SubtitleSSA ssa(this);
		if(ssa.load(filename))
		{
			filenameDocument = filename;
			return true;
		}
	}
	else if(filename.find(".srt") != Glib::ustring::npos)
	{
		SubtitleSRT srt(this);
		if(srt.load(filename))
		{
			filenameDocument = filename;
			return true;
		}
	}
	else
	{
		std::cerr << "Document::open > failed" << std::endl;
	}
	return false;
}

/*
 *
 */
bool Document::save(const Glib::ustring &filename, bool rename)
{
	if(rename)
		filenameDocument = filename;

	if(filename.find(".ssa") != Glib::ustring::npos)
	{
		SubtitleSSA ssa(this);
		return ssa.save(filename);
	}
	else if(filename.find(".srt") != Glib::ustring::npos)
	{
		SubtitleSRT srt(this);
		return srt.save(filename);
	}
	return false;
}
