/*
 *  subtitle editor
 *	http://kitone.free.fr/subtitleeditor/
 *
 *  Copyright @ 2005-2006, IDJAAD Djamel (aka Kitone)
 *
 *  Contact: kitone at free point fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author IDJAAD Djamel (kitone at free point fr)
 */

#include <gtkmm/filechooserdialog.h>
#include <gtkmm/entry.h>
#include <gtkmm/accelmap.h>
#include <gtkmm/checkbutton.h>
#include <gtkmm/main.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>
#include <gtkmm/toggleaction.h>

#include <gdk/gdkkeysyms.h>

#include "SubtitleEditor.h"
#include "SpellDialog.h"

#include <config.h>
#include "utility.h"
#include "SubtitleText.h"


/*
 *
 */
SubtitleEditor::SubtitleEditor(const Glib::ustring &filename_subtitle, const Glib::ustring &movie)
{
  set_title("subtitle editor");
  set_default_size(600,400);
  
  add(vbox);
  

  createMenuBar();  

	//m_refDocument = Glib::RefPtr<Document>(new Document);

	subtitleView = manage(new SubtitleView(getDocument()));
	
  vbox.pack_start(scrolledwindow);
  scrolledwindow.add(*subtitleView);
  scrolledwindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
 
  vbox.pack_start(statusbar, false,false);

  show_all();

  // on regarde si le repertoire ~/.subtitle_editor existe
  Glib::ustring path_se = Glib::build_filename(Glib::get_home_dir(), ".subtitle_editor");
  if(Glib::file_test(path_se, Glib::FILE_TEST_IS_DIR) == false)
  {
    Glib::ustring cmd = "mkdir " + path_se;
    system(cmd.c_str());
  }

	
	// lecture du sous titre
	if(filename_subtitle.size()>0)
  {
		if(getDocument()->open(filename_subtitle))
			setStatusbar(_("Open document :") + filename_subtitle);
  }


  
  // on va chercher la configuration clavier
  Glib::ustring path_se_accelmap = Glib::build_filename(path_se, "accelmap");
  
  Gtk::AccelMap::load(path_se_accelmap);
}

/*
 *
 */
SubtitleEditor::~SubtitleEditor()
{
  // on sauvegarde la configuration clavier
  Glib::ustring path_se = Glib::build_filename(Glib::get_home_dir(), ".subtitle_editor");
  Glib::ustring path_se_accelmap = Glib::build_filename(path_se, "accelmap");

  Gtk::AccelMap::save(path_se_accelmap);
}

/*
 *
 */
Document* SubtitleEditor::getDocument()
{
	return &m_refDocument;
}

/*
 * 
 */
void SubtitleEditor::setStatusbar(const Glib::ustring &txt)
{
  int context = statusbar.get_context_id("txt");
  //statusbar.remove_message(0, context);
  statusbar.push(txt, context);
}

/*
 *
 */
void SubtitleEditor::createMenuBar()
{
  Glib::RefPtr<Gtk::UIManager> m_refUIManager;
  Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;

  m_refActionGroup = Gtk::ActionGroup::create();

#define SIGNAL(act) sigc::bind<ACTION>(sigc::mem_fun(*this, &SubtitleEditor::on_action), act)
#define ADD_ACTION m_refActionGroup->add

  // FILE -------------
  ADD_ACTION(Gtk::Action::create("MenuFile", _("_File")));

  ADD_ACTION(Gtk::Action::create("New", Gtk::Stock::NEW),
    Gtk::AccelKey('n', Gdk::CONTROL_MASK), SIGNAL(NEW_SUBTITLE));

  ADD_ACTION(Gtk::Action::create("Open Subtitle", Gtk::Stock::OPEN, _("_Open Subtitle")), 
    Gtk::AccelKey('o', Gdk::CONTROL_MASK), SIGNAL(OPEN_SUBTITLE));

  ADD_ACTION(Gtk::Action::create("Save Subtitle", Gtk::Stock::SAVE, _("_Save Subtitle")), 
    Gtk::AccelKey('s', Gdk::CONTROL_MASK), SIGNAL(SAVE_SUBTITLE));

  ADD_ACTION(Gtk::Action::create("Save As Subtitle", Gtk::Stock::SAVE_AS, _("Save As Subtitle")),
    SIGNAL(SAVE_AS_SUBTITLE));

  ADD_ACTION(Gtk::Action::create("Properties", _("Properties")),
    SIGNAL(PROPERTIES));
	
	ADD_ACTION(Gtk::Action::create("Import Text", _("Import Text")),
    SIGNAL(IMPORT_TEXT));

  ADD_ACTION(Gtk::Action::create("Export Text", _("Export Text")),
    SIGNAL(EXPORT_TEXT));

  ADD_ACTION(Gtk::Action::create("Quit", Gtk::Stock::QUIT), 
    SIGNAL(QUIT));

	
	// SUBTITLE -------------
  ADD_ACTION(Gtk::Action::create("MenuSubtitle", _("_Subtitle")));
  
  ADD_ACTION(Gtk::Action::create("Edit Subtitle", Gtk::Stock::EDIT, _("_Edit Subtitle [DISABLE]")), 
    Gtk::AccelKey(GDK_Return, Gdk::MOD1_MASK),
    SIGNAL(EDIT_SUBTITLE));

  ADD_ACTION(Gtk::Action::create("Move Subtitle", Gtk::Stock::JUMP_TO, _("_Move Subtitle")),
    SIGNAL(MOVE_SUBTITLE));

  ADD_ACTION(Gtk::Action::create("Insert Before", Gtk::Stock::GOTO_TOP, _("Insert Before")), 
    Gtk::AccelKey(GDK_Insert, Gdk::CONTROL_MASK), 
    SIGNAL(INSERT_BEFORE_SUBTITLE));

  ADD_ACTION(Gtk::Action::create("Insert After", Gtk::Stock::GOTO_BOTTOM, _("Insert After")), 
    Gtk::AccelKey(GDK_Insert, (Gdk::ModifierType)0), 
    SIGNAL(INSERT_AFTER_SUBTITLE));

  ADD_ACTION(Gtk::Action::create("Delete", Gtk::Stock::DELETE), 
    Gtk::AccelKey(GDK_Delete, Gdk::CONTROL_MASK), 
    SIGNAL(DELETE_SUBTITLE));

  ADD_ACTION(Gtk::Action::create("Change FPS", Gtk::Stock::CONVERT, _("Change FPS")),
    SIGNAL(CHANGE_FPS));

  ADD_ACTION(Gtk::Action::create("Scale", Gtk::Stock::CONVERT, _("Scale")),
    SIGNAL(SCALE));

  // TOOLS  -------------
  ADD_ACTION(Gtk::Action::create("MenuTools", _("_Tools")));

  ADD_ACTION(Gtk::Action::create("Find", Gtk::Stock::FIND, _("Find")),
    Gtk::AccelKey('f', Gdk::CONTROL_MASK),
    SIGNAL(FIND));

  ADD_ACTION(Gtk::Action::create("Find Next", Gtk::Stock::FIND, _("Find Next")),
    Gtk::AccelKey('g', Gdk::CONTROL_MASK),
    SIGNAL(FIND_NEXT));

  ADD_ACTION(Gtk::Action::create("Find And Replace", Gtk::Stock::FIND_AND_REPLACE, _("Find And Replace")),
    Gtk::AccelKey('g', Gdk::SHIFT_MASK | Gdk::CONTROL_MASK),
    SIGNAL(FIND_AND_REPLACE));

  //ADD_ACTION(Gtk::Action::create("Remove Last Char", Gtk::Stock::DELETE, _("Remove Last Char")),
  //  SIGNAL(REMOVE_LAST_CHAR));

  ADD_ACTION(Gtk::Action::create("Remove Line Nul", Gtk::Stock::DELETE, _("Remove Line Nul")),
    SIGNAL(REMOVE_LINE_NUL));

  ADD_ACTION(Gtk::Action::create("Check Time", Gtk::Stock::YES, _("Check Time")),
    Gtk::AccelKey('c', Gdk::CONTROL_MASK), 
    SIGNAL(CHECK_TIME));

  ADD_ACTION(Gtk::Action::create("Spell Check", Gtk::Stock::SPELL_CHECK, _("Spell Check")),
    Gtk::AccelKey('c', Gdk::MOD1_MASK), 
    SIGNAL(SPELL_CHECK));

  ADD_ACTION(Gtk::Action::create("Set All End Time", _("Set All End Time")),
    SIGNAL(SET_ALL_END_TIME));

  ADD_ACTION(Gtk::Action::create("Check Error Encoding", _("Check Error Encoding")),
    SIGNAL(CHECK_ERROR_ENCODING));

  // VIDEO  -------------
  ADD_ACTION(Gtk::Action::create("MenuVideo", _("_Video")));

  ADD_ACTION(Gtk::Action::create("Open Movie", Gtk::Stock::OPEN, _("Open Movie")),
    Gtk::AccelKey('o', Gdk::SHIFT_MASK | Gdk::CONTROL_MASK),
    SIGNAL(OPEN_MOVIE));
 
  ADD_ACTION(Gtk::Action::create("Play Movie", Gtk::Stock::MEDIA_PLAY, _("_Play Movie")), 
    Gtk::AccelKey(GDK_space, Gdk::CONTROL_MASK),
    SIGNAL(PLAY_MOVIE));

  ADD_ACTION(Gtk::Action::create("Style Editor", _("Style Editor")),
    SIGNAL(STYLE_EDITOR));
	
  // OPTIONS  -------------
  ADD_ACTION(Gtk::Action::create("MenuOptions", _("_Options")));

  ADD_ACTION(Gtk::Action::create("MenuView", _("_View [TODO]")));
	  ADD_ACTION(Gtk::ToggleAction::create("Show Num", _("Show Num"), "", true), SIGNAL(SHOW_NUM));  
	  ADD_ACTION(Gtk::ToggleAction::create("Show Layer", _("Show Layer"), "", false), SIGNAL(SHOW_LAYER));  
	  ADD_ACTION(Gtk::ToggleAction::create("Show Start", _("Show Start"), "", true), SIGNAL(SHOW_START));  
	  ADD_ACTION(Gtk::ToggleAction::create("Show End", _("Show End"), "", true), SIGNAL(SHOW_END));  
	  ADD_ACTION(Gtk::ToggleAction::create("Show Style", _("Show Style"), "", true), SIGNAL(SHOW_STYLE));  
	  ADD_ACTION(Gtk::ToggleAction::create("Show Name", _("Show Name"), "", true), SIGNAL(SHOW_NAME));  
	  ADD_ACTION(Gtk::ToggleAction::create("Show Margin", _("Show Margin"), "", false), SIGNAL(SHOW_MARGIN));  
	  ADD_ACTION(Gtk::ToggleAction::create("Show Effect", _("Show Effect"), "", false), SIGNAL(SHOW_EFFECT));  
	  ADD_ACTION(Gtk::ToggleAction::create("Show Text", _("Show Text"), "", true), SIGNAL(SHOW_TEXT));  
	
  ADD_ACTION(Gtk::Action::create("Set View SRT", _("Set View SRT")), 
    SIGNAL(SET_VIEW_SRT));

  ADD_ACTION(Gtk::Action::create("Set View SSA", _("Set View SSA")), 
    SIGNAL(SET_VIEW_SSA));


	// HELP   -------------
  ADD_ACTION(Gtk::Action::create("MenuHelp", _("_Help")));
  ADD_ACTION(Gtk::Action::create("About", Gtk::Stock::HELP, _("_About")),
    SIGNAL(ABOUT));

#undef ADD_ACTION
#undef SIGNAL

  m_refUIManager = Gtk::UIManager::create();
  m_refUIManager->insert_action_group(m_refActionGroup);


  add_accel_group(m_refUIManager->get_accel_group());


  try
  {
    Glib::ustring ui_info =
        "<ui>"
        "  <menubar name='MenuBar'>"
        "    <menu action='MenuFile'>"
        "      <menuitem action='New'/>"
        "      <menuitem action='Open Subtitle'/>"
        "      <menuitem action='Save Subtitle'/>"
        "      <menuitem action='Save As Subtitle'/>"
        "      <separator/>"
        "      <menuitem action='Properties'/>"
        "      <separator/>"
				"      <menuitem action='Import Text'/>"
				"      <menuitem action='Export Text'/>"
        "      <separator/>"
        "      <menuitem action='Quit'/>"
        "    </menu>"
				"    <menu action='MenuSubtitle'>"
        "      <menuitem action='Edit Subtitle'/>"
        "      <menuitem action='Move Subtitle'/>"
        "      <separator/>"
        "      <menuitem action='Insert Before'/>"
        "      <menuitem action='Insert After'/>"
        "      <separator/>"
        "      <menuitem action='Delete'/>"
        "      <separator/>"
        "      <menuitem action='Change FPS'/>"
        "      <menuitem action='Scale'/>"
        "    </menu>"
        "    <menu action='MenuTools'>"
				"			 <menuitem action='Style Editor'/>"
        "      <separator/>"
				"      <menuitem action='Find'/>"
        "      <menuitem action='Find Next'/>"
        "      <menuitem action='Find And Replace'/>"
        "      <separator/>"
				"      <menuitem action='Remove Line Nul'/>"
        "      <separator/>"
				"      <menuitem action='Check Time'/>"
				"      <menuitem action='Spell Check'/>"
        "      <separator/>"
				"      <menuitem action='Set All End Time'/>"
				"			 <menuitem action='Check Error Encoding'/>"
        "    </menu>"
        "    <menu action='MenuVideo'>"
        "      <menuitem action='Open Movie'/>"
        "      <menuitem action='Play Movie'/>"
        "    </menu>"
				"		 <menu action='MenuOptions'>"
				"				<menu action='MenuView'>"
        "					<menuitem action='Show Num'/>"
        "					<menuitem action='Show Layer'/>"
        "					<menuitem action='Show Start'/>"
        "					<menuitem action='Show End'/>"
        "					<menuitem action='Show Style'/>"
        "					<menuitem action='Show Name'/>"
        "					<menuitem action='Show Margin'/>"
        "					<menuitem action='Show Effect'/>"
        "					<menuitem action='Show Text'/>"
				"				</menu>"
        "      <menuitem action='Set View SRT'/>"
        "      <menuitem action='Set View SSA'/>"
        "      <separator/>"
				"		 </menu>"
        "    <menu action='MenuHelp'>"
        "      <menuitem action='About'/>"
        "    </menu>"
        "  </menubar>"
        "</ui>";

    m_refUIManager->add_ui_from_string(ui_info);
  
    vbox.pack_start(*m_refUIManager->get_widget("/MenuBar"),false,false);
  }
  catch(const Glib::Error& ex)
  {
    std::cerr << _("building menubar failed: ") <<  ex.what();
    //exit();
  }
}


/*
 *
 */
void SubtitleEditor::on_action(ACTION action)
{
  switch(action)
  {
  // FILE
  case NEW_SUBTITLE:
    on_new_subtitle();
    break;
  case OPEN_SUBTITLE:
    on_open_subtitle();
    break;
  case SAVE_SUBTITLE:
    on_save_subtitle();
    break;
  case SAVE_AS_SUBTITLE:
    on_save_as_subtitle();
    break;
  case QUIT:
    on_quit();
    break;
  case IMPORT_TEXT:
    on_import_text();
    break;
  case EXPORT_TEXT:
    on_export_text();
    break;
	case PROPERTIES:
		on_properties();
		break;
	// VIEW
	case SET_VIEW_SRT:
		on_set_view_type_srt();
		break;
	case SET_VIEW_SSA:
		on_set_view_type_ssa();
		break;
  // SUBTITLE
  case MOVE_SUBTITLE:
    on_move_subtitle();
    break;
  case EDIT_SUBTITLE:
    on_edit_subtitle();
    break;
  case INSERT_BEFORE_SUBTITLE:
    on_insert_before_subtitle();
    break;
  case INSERT_AFTER_SUBTITLE:
    on_insert_after_subtitle();
    break;
  case DELETE_SUBTITLE:
    on_delete_subtitle();
    break;
  case CHANGE_FPS:
    on_change_fps();
    break;
  case SCALE:
    on_scale();
    break;
  // TOOLS
  case FIND:
    on_find();
    break;
  case FIND_NEXT:
    on_find_next();
    break;
  case FIND_AND_REPLACE:
    on_find_and_replace();
    break;
  case REMOVE_LINE_NUL:
    on_remove_line_nul();
    break;
  case CHECK_TIME:
    on_check_time();
    break;
  case SPELL_CHECK:
    on_spell_check();
    break;
  case SET_ALL_END_TIME:
    on_set_all_end_time();
    break;
	case CHECK_ERROR_ENCODING:
		on_check_error_encoding();
		break;
  // VIDEO
  case OPEN_MOVIE:
    on_open_movie();
    break;
  case PLAY_MOVIE:
    on_play_movie();
    break;
  // HELP
  case ABOUT:
    on_about();
    break;
	case STYLE_EDITOR:
		on_style_editor();
		break;
  default:
    break;
  }
}

/*
 *
 */
void SubtitleEditor::on_new_subtitle()
{
	getDocument()->clear();
	setStatusbar(_("New Document"));
}

/*
 *
 */
void SubtitleEditor::on_quit()
{
  //Gtk::MessageDialog dialog(*this, "Are you sure ?", false, Gtk::MESSAGE_QUESTION , Gtk::BUTTONS_YES_NO ,true);
  //if(dialog.run() == Gtk::RESPONSE_YES)
    Gtk::Main::quit();
}

/*
 *	on utilise l'extention du fichier pour ouvrir selon le type
 */
void SubtitleEditor::on_open_subtitle()
{
  Glib::ustring file ;
  m_dialogOpenSubtitle.show();
  
  if(m_dialogOpenSubtitle.run() == Gtk::RESPONSE_OK)
  {
		file = m_dialogOpenSubtitle.get_filename();
		
		if(getDocument()->open(file))
			setStatusbar(_("Open document :") + file);
		else
			setStatusbar(_("Open document failed :") + file);
  }
  m_dialogOpenSubtitle.hide();
}

/*
 *
 */
void SubtitleEditor::on_open_movie()
{
  m_dialogOpenMovie.show();
  if(m_dialogOpenMovie.run() == Gtk::RESPONSE_OK)
  {
    Glib::ustring file = m_dialogOpenMovie.get_filename();
    getDocument()->filenameMovie = file;
  }
  m_dialogOpenMovie.hide();
}

/*
 *
 */
void SubtitleEditor::on_save_subtitle()
{
  if(getDocument()->filenameDocument.size() > 0)
  {
		if(getDocument()->save(getDocument()->filenameDocument))
			setStatusbar(_("Save document :") + getDocument()->filenameDocument);
		else
			setStatusbar(_("Save document failed :") + getDocument()->filenameDocument);
	}
  else
  {
    on_save_as_subtitle();
  }
}

/*
 *
 */
void SubtitleEditor::on_save_as_subtitle()
{
  m_dialogSaveSubtitle.show();
  if(m_dialogSaveSubtitle.run() == Gtk::RESPONSE_OK)
  {
    Glib::ustring file = m_dialogSaveSubtitle.get_filename();

		if(getDocument()->save(file))
			setStatusbar(_("Save as document :") + file);
		else
			setStatusbar(_("Save as document failed :") + file);

	}
  m_dialogSaveSubtitle.hide();
}

/*
 *
 */
void SubtitleEditor::on_import_text()
{
  m_dialogImportText.show();
  if(m_dialogImportText.run() == Gtk::RESPONSE_OK)
  {
    Glib::ustring filename = m_dialogImportText.get_filename();
  
		getDocument()->clear();
		SubtitleText text(getDocument());
		if(text.load(filename))
			setStatusbar(_("Import Text :") + filename);
		else
			setStatusbar(_("Import Text failed :") + filename);
	}
  m_dialogImportText.hide();

}

/*
 *
 */
void SubtitleEditor::on_export_text()
{
	m_dialogExportText.show();
  if(m_dialogExportText.run() == Gtk::RESPONSE_OK)
  {
    Glib::ustring file = m_dialogExportText.get_filename();

		SubtitleText text(getDocument());
		if(text.save(file))
			setStatusbar(_("Export Text :") + file);
		else
			setStatusbar(_("Export Text failed :") + file);
	}
  m_dialogExportText.hide();  
	
}

/*
 *
 */
void SubtitleEditor::on_properties()
{
	DialogScriptProperties dialog(getDocument());
}

/*
 *
 */
void SubtitleEditor::on_play_movie()
{
  // on ecrit un fichier tmp pour les sous titres..
	Glib::ustring subtitle = Glib::build_filename(Glib::get_tmp_dir(), "subtitle." + getDocument()->getFormat());

	getDocument()->save(subtitle, false);
	
  
  long position = 0;

  // si il y a un subtitle de select 
  // alors on commence a partir de ça position moins 4 sec
  Gtk::TreeIter iter = subtitleView->getSelected();
  if(iter)
  {
    SubtitleColumnRecorder m_column;
    SubtitleTime time = SubtitleTime((*iter)[m_column.start]) - SubtitleTime(0,0,4,0);
    
    position = time.hours*3600 + time.mins*60 + time.secs;
  }

  // on lance MPlayer avec comme option l'affichage du temps (-osdlevel 2)
  gchar *cmd = g_strdup_printf(
    "mplayer \"%s\" -sub \"%s\" -ss %d -osdlevel 2",
      getDocument()->filenameMovie.c_str(), 
			subtitle.c_str(), position);
	
  Glib::spawn_command_line_async(cmd);

	std::cout << std::endl;
	std::cout << "SubtitleEditor execute: " << cmd << std::endl;

  g_free(cmd);
}

/*
 *
 */
void SubtitleEditor::on_move_subtitle()
{
  Gtk::TreeIter iter = subtitleView->getSelected();
  if(!iter)
    return;

  SubtitleColumnRecorder m_column;
	
	SubtitleTime start((*iter)[m_column.start]);

  m_dialogMoveSubtitle.set_value(start);
  m_dialogMoveSubtitle.show();

  if(m_dialogMoveSubtitle.run() == Gtk::RESPONSE_OK)
  {
    SubtitleTime diff = m_dialogMoveSubtitle.get_value() - start;
    getDocument()->m_subtitleModel->move_all((*iter)[m_column.num], diff.totalmsecs);
  }
  m_dialogMoveSubtitle.hide();
}



/*
 * edit le subtitle select
 */
void SubtitleEditor::on_edit_subtitle()
{
#warning "fixme > Subtitle"
/*	
  Gtk::TreeIter iter = subtitleView->getSelected();
  if(!iter)
  {
    // si le model est vide alors on cre un par defaut
    if(getDocument()->m_subtitleModel->getSize() == 0)
    {
      Gtk::TreeIter it = getDocument()->m_subtitleModel->append();
      Subtitle sub;
      getDocument()->m_subtitleModel->refresh(it, sub);
    }
    return;
  }

  SubtitleColumnRecorder m_column;
  Subtitle select = (*iter)[m_column.subtitle];

  m_dialogEditSubtitle.set_value(select);
  
  m_dialogEditSubtitle.show();
  
  if(m_dialogEditSubtitle.run() == Gtk::RESPONSE_OK)
  {
    select = m_dialogEditSubtitle.get_value();
    (*iter)[m_column.subtitle] = select;
    getDocument()->m_subtitleModel->refresh(iter, select);
  }

  m_dialogEditSubtitle.hide();
*/
}

/*
 *
 */
void SubtitleEditor::on_insert_before_subtitle()
{
  Gtk::TreeIter select = subtitleView->getSelected();
  if(!select)
  {
    // si le model est vide alors on cre un par defaut
    if(getDocument()->m_subtitleModel->getSize() == 0)
    {
      Gtk::TreeIter it = getDocument()->m_subtitleModel->append();
      subtitleView->select_and_set_cursor(it);
    }
    return;
  }

  SubtitleColumnRecorder m_column;

  SubtitleTime time_start;
  Gtk::TreeIter prev = getDocument()->m_subtitleModel->find_previous(select);
  
  if(prev)
  {
    time_start = SubtitleTime((*prev)[m_column.end]);
  }

  // on recupere le temps de depart pour s'en servir comme temps de fin
  SubtitleTime time_end = SubtitleTime((*select)[m_column.start]);
  
  
  Gtk::TreeIter new_iter = getDocument()->m_subtitleModel->insertBefore(select);
  subtitleView->select_and_set_cursor(new_iter);

	(*new_iter)[m_column.start]	= time_start.str();
	(*new_iter)[m_column.end]		= time_end.str();
}

/*
 *
 */
void SubtitleEditor::on_insert_after_subtitle()
{
  Gtk::TreeIter iter_select = subtitleView->getSelected();
  if(!iter_select)
  {
    // si le model est vide alors on cre un par defaut
    if(getDocument()->m_subtitleModel->getSize() == 0)
    {
      Gtk::TreeIter it = getDocument()->m_subtitleModel->append();
      subtitleView->select_and_set_cursor(it);
    }
    return;
  }

  SubtitleColumnRecorder m_column;
  
  SubtitleTime time_start, time_end;
  
  time_start = SubtitleTime((*iter_select)[m_column.end]);
  
  Gtk::TreeIter next = getDocument()->m_subtitleModel->find_next(iter_select);
  if(next)
  {
		time_end = SubtitleTime((*next)[m_column.start]);
  }
  else  // s'il n'y a pas de suivant par defaut on ajoute 2 secs
    time_end = time_start + SubtitleTime(0,0,2,0);
  
  Gtk::TreeIter new_iter = getDocument()->m_subtitleModel->insertAfter(iter_select);
  subtitleView->select_and_set_cursor(new_iter);
	
	(*new_iter)[m_column.start]	= time_start.str();
	(*new_iter)[m_column.end]		= time_end.str();
}

/*
 *
 */
void SubtitleEditor::on_delete_subtitle()
{
  Gtk::TreeIter iter_select = subtitleView->getSelected();
  if(!iter_select)
    return;
	
  Gtk::TreeIter next = getDocument()->m_subtitleModel->find_next(iter_select);
	Gtk::TreeIter prev = getDocument()->m_subtitleModel->find_previous(iter_select);


  getDocument()->m_subtitleModel->remove(iter_select);

	// select next line
	if(next)
  {
		subtitleView->select_and_set_cursor(next);
  }
	else if(prev)
	{
		subtitleView->select_and_set_cursor(prev);
	}
}

/*
 *
 */
void SubtitleEditor::on_change_fps()
{
  m_dialogChangeFPS.show();
  if(m_dialogChangeFPS.run() == Gtk::RESPONSE_OK)
  {
    double src = m_dialogChangeFPS.get_source();
    double dest= m_dialogChangeFPS.get_dest();

    if(src!=0 && dest!=0)
      getDocument()->m_subtitleModel->changeFPS(src,dest);
  }
  m_dialogChangeFPS.hide();
}

/*
 *
 */
void SubtitleEditor::on_scale()
{
	/*
  m_dialogScaleSubtitle.show();
  
  if(m_dialogScaleSubtitle.run() == Gtk::RESPONSE_OK)
  {
    SubtitleTime s1 = m_dialogScaleSubtitle.m_subtitleTimeSource1.get_value();
    SubtitleTime d1 = m_dialogScaleSubtitle.m_subtitleTimeDest1.get_value();
    SubtitleTime s2 = m_dialogScaleSubtitle.m_subtitleTimeSource2.get_value();
    SubtitleTime d2 = m_dialogScaleSubtitle.m_subtitleTimeDest2.get_value();
    
    getDocument()->m_subtitleModel->scale(s1, d1, s2, d2);
  }
  m_dialogScaleSubtitle.hide();
	*/
	DialogScaleSubtitle2 diag(getDocument()->m_subtitleModel);
	diag.show();
	diag.run();
}



/*
 * tools
 */
void SubtitleEditor::on_find()
{
  m_dialogFind.show();

  if(m_dialogFind.run() == Gtk::RESPONSE_OK)
  {
    last_text_find = m_dialogFind.getFindText();
    if(last_text_find.size() > 0)
    {
      Gtk::TreeIter start;
      Gtk::TreeIter iter_select = subtitleView->getSelected();
    
      if(iter_select)
				start = iter_select;
      else
				start = getDocument()->m_subtitleModel->getFirst();
    
      Gtk::TreeIter res = getDocument()->m_subtitleModel->find_text(start, last_text_find);
      if(res)
      {
				subtitleView->select_and_set_cursor(res);
      }
			else
				setStatusbar(_("Find > not found: ") + last_text_find);
    }
  }
  m_dialogFind.hide();
}

/*
 *
 */
void SubtitleEditor::on_find_next()
{
  if(last_text_find.size() == 0)
    return;

  Gtk::TreeIter start;
  Gtk::TreeIter iter_select = subtitleView->getSelected();
  
  if(iter_select)
    start = iter_select;
  else
    start = getDocument()->m_subtitleModel->getFirst();
  
  Gtk::TreeIter res = getDocument()->m_subtitleModel->find_text(start, last_text_find);
  if(res)
  {
    subtitleView->select_and_set_cursor(res);
  }
	else
		setStatusbar(_("Find > not found: ") + last_text_find);
}

/*
 *
 */
bool find_and_replace(Glib::ustring &source, 
		const Glib::ustring &str_find, 
		const Glib::ustring &str_replace)
{
  //g_return_if_fail(source.size());
  //g_return_if_fail(str_find.size());
  if(source.size()==0) return false;
  if(str_find.size()==0) return false;

  // on regarde si le txt rechercher ce trouve dans la source
  Glib::ustring::size_type pos = source.find(str_find, 0);

	// on regarde si il l'a trouver au moins une fois pour retrouner true
	if(pos == Glib::ustring::npos)
		return false;
	
	// tant qu'il est dans la source on le recherche
  while(pos != Glib::ustring::npos)
  {
    source.replace(pos, str_find.size(), str_replace);
    pos = source.find(str_find, pos);
  }

	return true;
}

/*
 *
 */
void SubtitleEditor::on_find_and_replace()
{
  SubtitleColumnRecorder m_column;
 
  m_dialogFindAndReplace.show();
  
  // on recupere le point de depart a partir de la selection
  Gtk::TreeIter iter_select = subtitleView->getSelected();
  Gtk::TreeIter select = getDocument()->m_subtitleModel->getFirst();
  //if(iter_select)
  //  start = iter_select;

  setStatusbar("");

  // on boucle
  while(int response = m_dialogFindAndReplace.run())
  {
    if(response == DialogFindAndReplace::CLOSE || response == Gtk::RESPONSE_CLOSE)
      break;
    else if(response == DialogFindAndReplace::FIND)
    {
      if(!select)
				select = getDocument()->m_subtitleModel->getFirst();
      else
      {
				Glib::ustring txt_find = m_dialogFindAndReplace.getFindText();
				select = getDocument()->m_subtitleModel->find_text(select, txt_find);

				if(select)
				{
          subtitleView->select_and_set_cursor(select);
				}
				else
					select = getDocument()->m_subtitleModel->getFirst();
       }
    }
    else if(response == DialogFindAndReplace::REPLACE)
    {
      if(select)
      {
				Glib::ustring txt_find = m_dialogFindAndReplace.getFindText();
				Glib::ustring txt_replace = m_dialogFindAndReplace.getReplaceText();

				Glib::ustring text = (*select)[m_column.text];
				
				if(find_and_replace(text, txt_find, txt_replace))
					(*select)[m_column.text] = text;
      }
    }
    else if(response == DialogFindAndReplace::REPLACE_ALL)
    {
      select = getDocument()->m_subtitleModel->getFirst();
      
      Glib::ustring txt_find = m_dialogFindAndReplace.getFindText();
      Glib::ustring txt_replace = m_dialogFindAndReplace.getReplaceText();

      while(select = getDocument()->m_subtitleModel->find_text(select, txt_find))
      {
				Glib::ustring txt_find = m_dialogFindAndReplace.getFindText();
				Glib::ustring txt_replace = m_dialogFindAndReplace.getReplaceText();

				Glib::ustring text = (*select)[m_column.text];
				
				if(find_and_replace(text, txt_find, txt_replace))
					(*select)[m_column.text] = text;
			}
    }
    else
      break;
  }
  
  m_dialogFindAndReplace.hide();
}


/*
 *
 */
void SubtitleEditor::on_remove_line_nul()
{
  SubtitleColumnRecorder m_column;
	Glib::ustring text;
	
  Gtk::TreeIter iter = getDocument()->m_subtitleModel->getFirst();
  for(; iter; )
  {
		text = (*iter)[m_column.text];
		
    if(text.size() == 0 || text == " ")
    {
      iter = getDocument()->m_subtitleModel->erase(iter);
    }
    else
      ++iter;
  }
}


/*
 *  on verifie la validiter du temps
 */
void SubtitleEditor::on_check_time()
{
  SubtitleColumnRecorder m_column;

  Gtk::TreeIter iter = getDocument()->m_subtitleModel->getFirst();

	SubtitleTime start, end;
  SubtitleTime prev_start, prev_end;
  
	for(; iter; ++iter)
  {
    //sub = (*iter)[m_column.subtitle];

		start = SubtitleTime((*iter)[m_column.start]);
		end = SubtitleTime((*iter)[m_column.end]);
		
    // on verifie que end et bien > a start
    if(end <= start)
    {
      subtitleView->get_selection()->select(iter);
      setStatusbar(_("Check Time Error : start > end"));
      return;
    }

    // on verifie qu'il est bien > au precedant
    if(start < prev_start)
    {
      subtitleView->get_selection()->select(iter);
      setStatusbar(_("Check Time Error : start < previous start"));
      return;
    }

    if(start < prev_end)
    {
      subtitleView->get_selection()->select(iter);
      setStatusbar(_("Check Time Error : start < previous end"));
      return;
    }

    prev_start	= start;
    prev_end	= end;
  }

  setStatusbar(_("Check Time OK!"));
}

/*
 *
 */
void SubtitleEditor::on_spell_check()
{
  setStatusbar(_("Spell check start"));
  SpellDialog spell;
  spell.execute(getDocument()->m_subtitleModel);
  setStatusbar(_("Spell check finish"));
}

/*
 *
 */
void SubtitleEditor::on_set_all_end_time()
{
  m_dialogSetAllEndTime.show();

  if(m_dialogSetAllEndTime.run()==Gtk::RESPONSE_OK)
  {
    SubtitleTime time = m_dialogSetAllEndTime.get_time();
    SubtitleColumnRecorder m_column;
    Gtk::TreeIter it = getDocument()->m_subtitleModel->getFirst();
  
    if(m_dialogSetAllEndTime.get_type() == DialogSetAllEndTime::ADD)
    {
      for(; it; ++it)
      {
				SubtitleTime it_time_end((*it)[m_column.end]);

				it_time_end = it_time_end + time;
				(*it)[m_column.end] = it_time_end.str();
				
				// verification il n'est pas trop long avec le prochain st
				Gtk::TreeIter it_next = getDocument()->m_subtitleModel->find_next(it);
				if(it_next)
				{
					SubtitleTime next_time_start = SubtitleTime((*it_next)[m_column.start]);
					
					if(it_time_end > next_time_start)
					{
						it_time_end = next_time_start;
					}
				}
	
				// mise a jour
				(*it)[m_column.end] = it_time_end.str();
      }
    }
    else // SET
    {
      for(; it; ++it)
      {
				SubtitleTime it_time_end((*it)[m_column.start]);

				it_time_end = it_time_end + time;
				(*it)[m_column.end] = it_time_end.str();
				
				// verification il n'est pas trop long avec le prochain st
				Gtk::TreeIter it_next = getDocument()->m_subtitleModel->find_next(it);
				if(it_next)
				{
					SubtitleTime next_time_start = SubtitleTime((*it_next)[m_column.start]);
					
					if(it_time_end > next_time_start)
					{
						it_time_end = next_time_start;
					}
				}
	
				// mise a jour
				(*it)[m_column.end] = it_time_end.str();
			}
    }
  }
  
  m_dialogSetAllEndTime.hide();
  
}

/*
 *	verifie dans une table des erreurs d'encodings.
 */
void SubtitleEditor::on_check_error_encoding()
{
	gchar* array[][2]={
		{"\302\234", "oe"},
		{"\302\222", "'"},
		{"\302\205", "."}, // ??
		{NULL,NULL}};

  SubtitleColumnRecorder m_column;

	bool state=false;

	Gtk::TreeIter iter = getDocument()->m_subtitleModel->getFirst();
	for(; iter; ++iter)
	{
		state = false;
		
		Glib::ustring text = (*iter)[m_column.text];
		
		for(unsigned int i=0; array[i][0] != NULL; ++i)
		{
			if(find_and_replace(text, array[i][0], array[i][1]))
			{
				state = true;
			}
		}

		if(state)
		{
			(*iter)[m_column.text] = text;
		}

  }
}


/*
 *
 */
void SubtitleEditor::on_about()
{
  DialogAbout about(*this);
  about.show_all();
  about.run();
}

/*
 *
 */
void SubtitleEditor::on_style_editor()
{
	DialogStyleEditor editor(getDocument());
}

/*
 *
 */
void SubtitleEditor::on_set_view_type_srt()
{
	subtitleView->set_view_type("srt");
}

/*
 *
 */
void SubtitleEditor::on_set_view_type_ssa()
{
	subtitleView->set_view_type("ssa");
}

