#ifndef _SubtitleEditor_h
#define _SubtitleEditor_h

/*
 *  subtitle editor
 *  Copyright @ 2005-2006, IDJAAD Djamel (aka Kitone)
 *
 *  Contact: kitone at free point fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author IDJAAD Djamel (kitone at free point fr)
 */

#include <iostream>
#include <gtkmm/window.h>
#include <gtkmm/table.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/box.h>
#include <gtkmm/spinbutton.h>
#include <gtkmm/dialog.h>
#include <gtkmm/label.h>
#include <gtkmm/uimanager.h>
#include <gtkmm/actiongroup.h>
#include <gtkmm/statusbar.h>

#include "Document.h"
#include "Dialog.h"
#include "SubtitleView.h"


enum ACTION
{
  // FILE
  NEW_SUBTITLE,
  OPEN_SUBTITLE,
  SAVE_SUBTITLE,
  SAVE_AS_SUBTITLE,
  IMPORT_TEXT,
  EXPORT_TEXT,
  QUIT,
	PROPERTIES,
	// OPTIONS
	// MenuVIEW
	SHOW_NUM,
	SHOW_LAYER,
	SHOW_START,
	SHOW_END,
	SHOW_STYLE,
	SHOW_NAME,
	SHOW_MARGIN,
	SHOW_EFFECT,
	SHOW_TEXT,
	

	SET_VIEW_SRT,
	SET_VIEW_SSA,
  // SUBTITLE
  MOVE_SUBTITLE,
  EDIT_SUBTITLE,
  INSERT_BEFORE_SUBTITLE,
  INSERT_AFTER_SUBTITLE,
  DELETE_SUBTITLE,
  CHANGE_FPS,
  SCALE,
  // TOOL,
  FIND,
  FIND_NEXT,
  FIND_AND_REPLACE,
  REMOVE_LINE_NUL,
  CHECK_TIME,
  SPELL_CHECK,
  SET_ALL_END_TIME,
	CHECK_ERROR_ENCODING,
  // MOVIE
  OPEN_MOVIE,
  PLAY_MOVIE,

	STYLE_EDITOR,

  // HELP
  ABOUT,
  ACTION_NONE
};


class SubtitleEditor : public Gtk::Window
{
public:
  SubtitleEditor(const Glib::ustring &srt, const Glib::ustring &movie);
  ~SubtitleEditor();
  
  void setStatusbar(const Glib::ustring &txt);
  void createMenuBar();
  
	/*
	 *
	 */
	Document* getDocument();

	
	/*
   *  callback utiliser pour declancher une action
   */
  void on_action(ACTION action);

  // file
  void on_new_subtitle();
  void on_quit();
  void on_open_subtitle();
  void on_save_subtitle();
  void on_save_as_subtitle();
	
	void on_properties();

  void on_import_text();
  void on_export_text();
	// view
	void on_set_view_type_srt();
	void on_set_view_type_ssa();

  // subtitle
  void on_edit_subtitle();
  void on_move_subtitle();
  void on_insert_before_subtitle();
  void on_insert_after_subtitle();
  void on_delete_subtitle();
  void on_change_fps();
  void on_scale();
  //  tools
  void on_find();
  void on_find_next();
  void on_find_and_replace();
  void on_remove_line_nul();
  void on_check_time();
  void on_spell_check();
  void on_set_all_end_time();
	void on_check_error_encoding();
  // movie
  void on_open_movie();
  void on_play_movie();
  // help
  void on_about();

	void on_style_editor();
  
protected:
	//Glib::RefPtr<Document>	m_refDocument;
	Document	m_refDocument;

  Gtk::VBox						vbox;
  SubtitleView*				subtitleView;
  Gtk::ScrolledWindow	scrolledwindow;
  Gtk::Statusbar			statusbar;

  // garde en memoire le dernier texte rechercher
  Glib::ustring		last_text_find;

  DialogFind		m_dialogFind;
  DialogFindAndReplace	m_dialogFindAndReplace;

  //
  DialogOpenSubtitle	m_dialogOpenSubtitle;
  DialogSaveSubtitle	m_dialogSaveSubtitle;
  DialogOpenMovie			m_dialogOpenMovie;
  DialogImportText		m_dialogImportText;
  DialogExportText		m_dialogExportText;

  DialogEditSubtitle	m_dialogEditSubtitle;
  DialogMoveSubtitle	m_dialogMoveSubtitle;
  DialogScaleSubtitle m_dialogScaleSubtitle;
  DialogChangeFPS			m_dialogChangeFPS;
  DialogSetAllEndTime	m_dialogSetAllEndTime;
};


#endif//_SubtitleEditor_h
