/*
 *  subtitle editor
 *	http://kitone.free.fr/subtitleeditor/
 *
 *  Copyright @ 2005-2006, IDJAAD Djamel (aka Kitone)
 *
 *  Contact: kitone at free point fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author IDJAAD Djamel (kitone at free point fr)
 */

#include "SubtitleSRT.h"
#include <iostream>
#include <fstream>
#include <glibmm/ustring.h>
#include "utility.h"

/*
 *
 */
SubtitleSRT::SubtitleSRT(Document* doc)
:m_model(doc->m_subtitleModel)
{
}

/*
 *
 */
SubtitleSRT::~SubtitleSRT()
{
}


/*
 *
 */
bool SubtitleSRT::load(const Glib::ustring &filename)
{
	std::ifstream file(filename.c_str());
	if(!file)
	{
		std::cerr << "SubtitleSRT::load > error open : " << filename << std::endl;
		return false;
	}

	SubtitleColumnRecorder column;

	std::string line;
	while(!file.eof() && std::getline(file, line))
	{
		unsigned int num = 0;

		// lecture du numero
		if(sscanf(line.c_str(), "%d", &num) > 0)
		{
			// lecture du temps
			if(std::getline(file, line))
			{
				int start[4], end[4];

				if(sscanf(line.c_str(), "%d:%d:%d,%d --> %d:%d:%d,%d",
							&start[0], &start[1], &start[2], &start[3], 
							&end[0], &end[1], &end[2], &end[3]) == 8)
				{
					Glib::ustring text;
					int count = 0; // permet de compter le nombre de ligne

					while(std::getline(file, line))
					{
						line = check_end_char(convert_to_utf8(line));

						if(line.size()==0)
							break;
						else
						{
							if(count > 0) // s'il y a deja une ligne alors on utlise le | pour le retour a la ligne
								text += "|";

							text += line;
							++count;
						}
					}

					Gtk::TreeIter iter = m_model->append();

					(*iter)[column.text]	= text;
					(*iter)[column.start] = (SubtitleTime(start[0], start[1], start[2], start[3]).str());
					(*iter)[column.end]		= (SubtitleTime(end[0], end[1], end[2], end[3]).str());
				}
			}
		}
	}

	file.close();

	return true;
}

/*
 *
 */
bool SubtitleSRT::save(const Glib::ustring &filename)
{
	std::ofstream file(filename.c_str());
	if(!file)
	{
		std::cerr << "SubtitleSRT::save > error open : " << filename << std::endl;
		return false;
	}

	SubtitleColumnRecorder column;
	Glib::ustring text;
	
	Gtk::TreeNodeChildren rows = m_model->children();
	for(Gtk::TreeIter it = rows.begin(); it; ++it)
	{
		file << (*it)[column.num] << std::endl;
		file << (*it)[column.start] << " --> " << (*it)[column.end] << std::endl;

		text = (*it)[column.text];

		Glib::ustring::size_type pos=0;

		while((pos = text.find('|', pos)) != Glib::ustring::npos)
		{
			text.replace(pos, 1, "\n");
		}

		file << text << std::endl;
		file << std::endl;
	}
	
	file.close();
	return true;
}
