/*
 *  subtitle editor
 *	http://kitone.free.fr/subtitleeditor/
 *
 *  Copyright @ 2005-2006, IDJAAD Djamel (aka Kitone)
 *
 *  Contact: kitone at free point fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author IDJAAD Djamel (kitone at free point fr)
 */

#include "SubtitleSSA.h"
#include <map>
#include <sstream>
#include <iostream>
#include <fstream>
#include <string>

Glib::ustring convert_to_utf8(const std::string &src);
Glib::ustring check_end_char(const Glib::ustring &str);

/*
 * clean source....
 */


/*
 *	hack !
 */
Glib::ustring clean_style_name(const Glib::ustring &name)
{
	Glib::ustring str = name;
	Glib::ustring::size_type n = str.find('*');
	if(n != Glib::ustring::npos)
		str.erase(n,1);

	return str;
}

/*
 *
 */
bool str2bool(const std::string &str)
{
	// ???? valide ????
	//
//	if(str == "-1")
//		return true;
//else
	if(str == "0")
		return false;
	else
		return true;
	
	return false;
}

/*
 *
 */
int str2int(const std::string &str)
{
	std::istringstream s(str);
	int i = 0;
	s >> i;

	return i;
}

/*
 *	recupere dans un tableau le format a partir de la line
 */
std::vector<std::string> build_format(const std::string &text, int column=-1, bool remove_space=false)
{
#warning "todo essayer a!"
	/*
	std::istringstream iss(text);
	std::string word;
	while( std::getline(iss, word, ',') )
	{
		std::cout << "word:" << word << std::endl;
	}
	*/

	
	std::string line;

	if(remove_space)
	{
		for(unsigned int i=7; i<text.size(); ++i)
		{
			if(text[i]==' ')
				;
			else
				line+=text[i];
		}
	}
	else
		line = text;

	std::vector<std::string> array;

	std::string::size_type s=0, e=0;

	unsigned int i=0;
	do
	{
		if(column > 0 && i+1 == column)
		{
			array.push_back(line.substr(s,std::string::npos));
			break;
		}
		e = line.find(",", s);
		
		array.push_back(line.substr(s,e-s));

		s=e+1;

		++i;
	}while(e != std::string::npos);

	return array;
}






/*
 *	construtor
 */
SubtitleSSA::SubtitleSSA(Document* doc)
:m_model(doc->m_subtitleModel), m_style_model(doc->m_styleModel), m_scriptInfo(&doc->m_scriptInfo)
{
}

/*
 *
 */
SubtitleSSA::~SubtitleSSA()
{
}

/*
 *	read subtitle file
 */
bool SubtitleSSA::load(const Glib::ustring &filename)
{
	std::ifstream file(filename.c_str());
	if(!file)
	{
		std::cerr << "SubtitleSSA::load > error open : " << filename << std::endl;
		return false;
	}

	std::string line;
	
	while(!file.eof() && std::getline(file, line))
	{
		if(line.find("[Script Info]") != std::string::npos)
		{
			readScripInfo(file);
		}
		else if(line.find("[V4 Styles]") != std::string::npos)
		{
			readStyles(file);
		}
		else if(line.find("[Events]") != std::string::npos)
		{
			readEvents(file);
		}
	}

	file.close();

	return true;
}

bool SubtitleSSA::save(const Glib::ustring &filename)
{
	std::ofstream file(filename.c_str());
	if(!file)
		return false;

	// ScriptInfo
	{
		file << "[Script Info]" << std::endl;
		file << "; This script is create by subtitle editor" << std::endl;
		file << "; http://kitone.free.fr/subtitleeditor/" << std::endl;
	
#define CHECK(label, key) if(m_scriptInfo->key.size() > 0) { file << label << m_scriptInfo->key << std::endl; }

		CHECK("Title: ", Title);
		CHECK("Original Script: ", OriginalScript);
		CHECK("Original Translation: ", OriginalTranslation);
		CHECK("Original Editing: ", OriginalEditing);
		CHECK("Synch Point: ", SynchPoint);
		CHECK("Script Updated By: ", ScriptUpdatedBy);
		CHECK("Update Details: ", UpdateDetails);
		CHECK("ScriptType: ", ScriptType);
		CHECK("Collisions: ", Collisions);
		CHECK("PlayResX: ", PlayResX);
		CHECK("PlayResY: ", PlayResY);
		CHECK("Timer: ", Timer);
		CHECK("Style: ", Style);
		CHECK("Dialogue: ", Dialogue);
		CHECK("Comment: ", Comment);
		CHECK("Picture: ", Picture);
		CHECK("Sound: ", Sound);
		CHECK("Movie: ", Movie);
		
#undef CHECK

		file << std::endl;
	}

	
	// Style
	{
		// alignment utiliser par SSA
		// 5  -  6 -  7
		// 9  - 10 - 11
		// 1  -  2 -  3
		std::map<unsigned int, unsigned int> alignment_map;
		alignment_map[1] = 1;
		alignment_map[2] = 2;
		alignment_map[3] = 3;
		alignment_map[4] = 9;
		alignment_map[5] =10;
		alignment_map[6] =11;
		alignment_map[7] = 5;
		alignment_map[8] = 6;
		alignment_map[9] = 7;

		StyleColumnRecorder column;
		
		file << "[V4 Styles]" << std::endl;

		file << "Format: "
				<< "Name, "
				<< "Fontname, "
				<< "Fontsize, "
				<< "PrimaryColour, "
				<< "SecondaryColour, "
				<< "TertiaryColour, "
				<< "BackColour, "
				<< "Bold, "
				<< "Italic, "
				<< "BorderStyle, "
				<< "Outline, "
				<< "Shadow, "
				<< "Alignment, "
				<< "MarginL, "
				<< "MarginR, "
				<< "MarginV, "
				<< "AlphaLevel, "
				<< "Encoding" << std::endl;

		Gtk::TreeNodeChildren rows = m_style_model->children();
		
		for(Gtk::TreeIter it = rows.begin(); it; ++it)
		{
			std::ostringstream oss;

#warning "utiliser -1 au lieu de 1 ?"

			oss << "Style: " 
				<< (*it)[column.name] << "," 
				<< (*it)[column.font_name] << "," 
				<< (*it)[column.font_size] << "," 
				<< (*it)[column.primary_colour] << "," 
				<< (*it)[column.secondary_colour] << "," 
				<< (*it)[column.outline_colour] << "," 
				<< (*it)[column.shadow_colour] << "," 
				<< (*it)[column.bold] << "," 
				<< (*it)[column.italic] << "," 
				<< (*it)[column.border_style] << "," 
				<< (*it)[column.outline] << "," 
				<< (*it)[column.shadow] << "," 
				<< alignment_map[ (*it)[column.alignment] ] << "," 
				<< (*it)[column.margin_l] << "," 
				<< (*it)[column.margin_r] << "," 
				<< (*it)[column.margin_v] << "," 
				<< (*it)[column.alpha_level] << "," 
				<< (*it)[column.encoding] << std::endl; 
				
			file << oss.str();
		}

		file << std::endl;
	}

	// Event
	{
		file << "[Events]" << std::endl;
		// format:
		file << "Format: " <<
			"Marked, " <<
			"Start, " <<
			"End, " <<
			"Style, " <<
			"Name, " <<
			"MarginL, " <<
			"MarginR, " <<
			"MarginV, " <<
			"Effect, " <<
			"Text" << std::endl;
			
		// dialog:
		SubtitleColumnRecorder column;
		Gtk::TreeNodeChildren rows = m_model->children();
		
		for(Gtk::TreeIter it = rows.begin(); it; ++it)
		{
			std::ostringstream oss;

			oss << "Dialogue: "
				<< "Marked=" << 0/*(*it)[column.flag]*/ << "," 
				<< convert_time_for_save((*it)[column.start]) << "," // convertir , en .
				<< convert_time_for_save((*it)[column.end]) << "," // convertir , en .
				<< (*it)[column.style] << ","
				<< (*it)[column.name] << ","
				<< (*it)[column.marginL] << ","
				<< (*it)[column.marginR] << ","
				<< (*it)[column.marginV] << ","
				<< (*it)[column.effect] << ","
				<< (*it)[column.text] << std::endl;

			file << oss.str();
		}
	}
	
	file.close();
	return true;
}

/*
 *	READ BLOCK
 */
	
/*
 *
 */
bool SubtitleSSA::readScripInfo(std::ifstream &file)
{
	std::cout << std::endl <<"SubtitleSSA::readScripInfo" << std::endl;
	//return true;
	
	std::string line;
	while(!file.eof() && std::getline(file, line))
	{
		if(line.size() < 3)
		{
			//std::cout << "line:" << line << ":" << std::endl;
			return true;
		}
		/*
		if(std::string::size_type n=line.find("Title: ") != std::string::npos)
		{
			line.erase(0,);
		}
		*/

#define CHECK(label, key) \
		if(line.find(label) != std::string::npos) \
		{	line.erase(0, (std::string(label)).size()); m_scriptInfo->key = check_end_char(convert_to_utf8(line)); }
//		else std::cerr << "CHECK not found: " << label << std::endl;

		CHECK("Title: ", Title);
		CHECK("Original Script: ", OriginalScript);
		CHECK("Original Translation: ", OriginalTranslation);
		CHECK("Original Editing: ", OriginalEditing);
		CHECK("Synch Point: ", SynchPoint);
		CHECK("Script Updated By: ", ScriptUpdatedBy);
		CHECK("Update Details: ", UpdateDetails);
		CHECK("ScriptType: ", ScriptType);
		CHECK("Collisions: ", Collisions);
		CHECK("PlayResX: ", PlayResX);
		CHECK("PlayResY: ", PlayResY);
		CHECK("Timer: ", Timer);
		CHECK("Style: ", Style);
		CHECK("Dialogue: ", Dialogue);
		CHECK("Comment: ", Comment);
		CHECK("Picture: ", Picture);
		CHECK("Sound: ", Sound);
		CHECK("Movie: ", Movie);

#undef CHECK
	}
	return true;
}

/*
 *	read Style
 */
bool SubtitleSSA::readStyles(std::ifstream &file)
{
	std::cout << std::endl <<"SubtitleSSA::readStyles" << std::endl;
	
	std::vector<std::string> formats;
	std::map<std::string, unsigned int> map; 
	
	std::string line;

	// alignment utiliser par SSA
  // 5  -  6 -  7
	// 9  - 10 - 11
	// 1  -  2 -  3
	std::map<unsigned int, unsigned int> alignment_map;
	alignment_map[1] = 1;
	alignment_map[2] = 2;
	alignment_map[3] = 3;
	alignment_map[9] = 4;
	alignment_map[10]= 5;
	alignment_map[11]= 6;
	alignment_map[5] = 7;
	alignment_map[6] = 8;
	alignment_map[7] = 9;
	
	while(!file.eof() && std::getline(file, line))
	{
		if(line.size() < 3)
			break;//return true;

		if(line.find("Style: ") != std::string::npos)
		{
			StyleColumnRecorder column;
			// on donne la ligne sans "Style: " = size - 7
			std::vector<std::string> fmt = build_format(line.substr(7, line.size()-7), formats.size(), false);

			Gtk::TreeIter it = m_style_model->append();

			// on supprime '*' si il existe
			(*it)[column.name]							= clean_style_name( fmt[ map["Name"] ] );
			
			(*it)[column.font_name]					= fmt[ map["Fontname"] ];
			(*it)[column.font_size]					= str2int( fmt[ map["Fontsize"] ] );
	
			(*it)[column.primary_colour]		= str2int( fmt[ map["PrimaryColour"] ]);
			(*it)[column.secondary_colour]	= str2int( fmt[ map["SecondaryColour"] ]);
			(*it)[column.outline_colour]		= str2int( fmt[ map["TertiaryColour"] ]);
			(*it)[column.shadow_colour]			= str2int( fmt[ map["BackColour"] ]);

			(*it)[column.bold]							= str2bool( fmt[ map["Bold"] ] );
			(*it)[column.italic]						= str2bool( fmt[ map["Italic"] ] );
			(*it)[column.border_style]			= str2bool( fmt[ map["BorderStyle"] ] );
			(*it)[column.outline]						= str2bool( fmt[ map["Outline"] ] );

			(*it)[column.shadow]						= str2int( fmt[ map["Shadow"] ]);
			(*it)[column.alignment]					= alignment_map[ str2int( fmt[ map["Alignment"] ]) ];
			(*it)[column.margin_l]					= str2int( fmt[ map["MarginL"] ]);
			(*it)[column.margin_r]					= str2int( fmt[ map["MarginR"] ]);
			(*it)[column.margin_v]					= str2int( fmt[ map["MarginV"] ]);
			
			(*it)[column.alpha_level]				= str2int( fmt[ map["AlphaLevel"] ]);
			(*it)[column.encoding]					= str2int( fmt[ map["Encoding"] ]);
	
		}
		else if(line.find("Format: ") != std::string::npos)
		{
			formats = build_format(line, -1, true);

			for(unsigned int i=0; i<formats.size(); ++i)
				map[formats[i]] = i;
		}
		
	}
	return true;
}

/*
 *
 */
bool SubtitleSSA::readEvents(std::ifstream &file)
{
	std::cout << std::endl <<"SubtitleSSA::readEvents" << std::endl;
	
	unsigned int num = 1;
	
	std::string line;
	while(!file.eof() && std::getline(file, line))
	{
		if(line.size() < 3)
			return true;

		if(std::string::size_type n=line.find("Dialogue: ") != std::string::npos)
		{
			line.erase(0,10);

			std::vector<std::string> array = build(line, 10);

			Gtk::TreeIter it = m_model->append();
			SubtitleColumnRecorder column;
			
			(*it)[column.num] = num;
			// marked/layer
			(*it)[column.start]		= convert_time_for_subtitleeditor(array[1]);
			(*it)[column.end]			= convert_time_for_subtitleeditor(array[2]);

			if(std::string::size_type p=array[3].find("*") != std::string::npos)
				array[3].erase(p,1);

			(*it)[column.style]		= array[3];//clean_style_name( array[3] );
			(*it)[column.name]		= array[4];
			
			(*it)[column.marginL]	= array[5];
			(*it)[column.marginR]	= array[6];
			(*it)[column.marginV]	= array[7];
			(*it)[column.effect]	= array[8];

			(*it)[column.text]		= check_end_char(convert_to_utf8(array[9]));

			++num;
		}
	}
	return true;
}


/*
 *
 */
std::vector< std::string > SubtitleSSA::build(const std::string &line, unsigned int column)
{
	std::vector< std::string > array;

	std::string::size_type s=0, e=0;

	do
	{
		if(column > 0 && array.size()+1 == column)
		{
			array.push_back(line.substr(s,std::string::npos));
			break;
		}

		e = line.find(",", s);
		
		array.push_back(line.substr(s,e-s));

		s=e+1;
	}while(e != std::string::npos);
	return array;
}


/*
 *	convertir le temps utiliser par subtitle editor en tant valide pour le format SSA
 *	h:m:s,ms -> h:m:s.ms
 */
Glib::ustring SubtitleSSA::convert_time_for_save(const Glib::ustring &time)
{
	Glib::ustring str = time;
	Glib::ustring::size_type f = str.find(',');
	str.replace(f, 1, ".");
	return str;
}

/*
 *	hours:mins:secs.msecs -> hours:mins:secs,msecs
 */
Glib::ustring SubtitleSSA::convert_time_for_subtitleeditor(const Glib::ustring &text)
{
  struct stime
  {
    int hours;
    int mins;
    int secs;
    int msecs;
  };

  stime t1;
  
  int res = sscanf(text.c_str(), "%d:%d:%d.%d",
    &t1.hours, &t1.mins, &t1.secs, &t1.msecs);
  
  if(res != 4)
    return "error";
  
  SubtitleTime time(t1.hours, t1.mins, t1.secs, t1.msecs);

  return getTime2String(time);
}
