/*
 *  subtitle editor
 *  Copyright @ 2005-2006, IDJAAD Djamel (aka Kitone)
 *
 *  Contact: kitone at free point fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author IDJAAD Djamel (kitone at free point fr)
 */

#include <iostream>
#include "SubtitleView.h"
#include "utility.h"
#include <gtkmm/cellrenderercombo.h>


SubtitleView::SubtitleView(Document* doc)
:m_refDocument(doc)
{
  set_model(doc->m_subtitleModel);

	m_subtitleModel = doc->m_subtitleModel;
	m_styleModel = doc->m_styleModel;
	
  createColumnNum();
	createColumnLayer();
  createColumnStart();
  createColumnEnd();
	createColumnStyle();
	createColumnName();
	
	createColumnMarginL();
	createColumnMarginR();
	createColumnMarginV();
	
  createColumnText();

  set_rules_hint(true);

	set_view_type("ssa");
}


/*
 *  create columns
 */
void SubtitleView::createColumnNum()
{
  Gtk::TreeViewColumn* column = NULL;
  Gtk::CellRendererText* renderer = NULL;
  
  column = manage(new Gtk::TreeViewColumn(_("num")));
  renderer = manage(new Gtk::CellRendererText);
  
  column->pack_start(*renderer, false);
  column->add_attribute(renderer->property_text(), m_column.num);
  
  append_column(*column);
  //append_column_numeric_editable("num", m_column.num, "%d");
  renderer->property_editable() = false;

	m_columns["num"] = column;
}

/*
 *
 */
void SubtitleView::createColumnLayer()
{
  Gtk::TreeViewColumn* column = NULL;
  Gtk::CellRendererText* renderer = NULL;
  
  column = manage(new Gtk::TreeViewColumn(_("layer")));
  renderer = manage(new Gtk::CellRendererText);
  
  column->pack_start(*renderer, false);
  column->add_attribute(renderer->property_text(), m_column.layer);

  renderer->property_editable() = true;
  renderer->signal_edited().connect(
    sigc::mem_fun(*this, &SubtitleView::on_edited_layer));

  append_column(*column);

	m_columns["layer"] = column;
}

/*
 *
 */
void SubtitleView::createColumnStart()
{
  Gtk::TreeViewColumn* column = NULL;
  Gtk::CellRendererText* renderer = NULL;
  
  column = manage(new Gtk::TreeViewColumn(_("start")));
  renderer = manage(new Gtk::CellRendererText);
  
  column->pack_start(*renderer, false);
  column->add_attribute(renderer->property_text(), m_column.start);
  
  renderer->property_editable() = true;
  renderer->signal_edited().connect(
    sigc::mem_fun(*this, &SubtitleView::on_edited_start));

  append_column(*column);

	m_columns["start"] = column;
}

/*
 *
 */
void SubtitleView::createColumnEnd()
{
  Gtk::TreeViewColumn* column = NULL;
  Gtk::CellRendererText* renderer = NULL;
  
  column = manage(new Gtk::TreeViewColumn(_("end")));
  renderer = manage(new Gtk::CellRendererText);
  
  column->pack_start(*renderer, false);
  column->add_attribute(renderer->property_text(), m_column.end);
  
  renderer->property_editable() = true;
  renderer->signal_edited().connect(
    sigc::mem_fun(*this, &SubtitleView::on_edited_end));

  append_column(*column);

	m_columns["end"] = column;
}

/*
 *
 */
void SubtitleView::createColumnStyle()
{
  Gtk::TreeViewColumn* column = NULL;
  Gtk::CellRendererCombo* renderer = NULL;
  
  column = manage(new Gtk::TreeViewColumn(_("style")));
  renderer = manage(new Gtk::CellRendererCombo);
  
  column->pack_start(*renderer, false);
	column->add_attribute(renderer->property_text(), m_column.style);
  
	renderer->property_model() =  m_styleModel;
  renderer->property_text_column() = 0;
	renderer->property_editable() = true;
	renderer->property_has_entry() = false;
	
	renderer->signal_edited().connect(
    sigc::mem_fun(*this, &SubtitleView::on_edited_style));

  append_column(*column);

	m_columns["style"] = column;
}

/*
 *
 */
void SubtitleView::createColumnName()
{
	/*
  Gtk::TreeViewColumn* column = NULL;
  Gtk::CellRendererCombo* renderer = NULL;
  
  column = manage(new Gtk::TreeViewColumn(_("name")));
  renderer = manage(new Gtk::CellRendererCombo);
  
  column->pack_start(*renderer, false);
  column->add_attribute(renderer->property_text(), m_column.name);
	
	renderer->property_model() =  m_refDocument->m_nameModel;
  renderer->property_text_column() = 0;
	renderer->property_editable() = true;
	renderer->property_has_entry() = true;


  renderer->signal_edited().connect(
    sigc::mem_fun(*this, &SubtitleView::on_edited_name));
	*/
  Gtk::TreeViewColumn* column = NULL;
  Gtk::CellRendererText* renderer = NULL;
  
  column = manage(new Gtk::TreeViewColumn(_("name")));
  renderer = manage(new Gtk::CellRendererText);
  
  column->pack_start(*renderer, false);
  column->add_attribute(renderer->property_text(), m_column.name);
	
	renderer->property_editable() = true;

  renderer->signal_edited().connect(
    sigc::mem_fun(*this, &SubtitleView::on_edited_name));
	
  append_column(*column);

	m_columns["name"] = column;
}

/*
 *
 */
void SubtitleView::createColumnText()
{
  Gtk::TreeViewColumn* column = NULL;
  Gtk::CellRendererText* renderer = NULL;
  
  column = manage(new Gtk::TreeViewColumn(_("text")));
  renderer = manage(new Gtk::CellRendererText);
  
  column->pack_start(*renderer, false);
  column->add_attribute(renderer->property_text(), m_column.text);
  
  append_column(*column);
  
  renderer->property_editable() = true;
  renderer->signal_edited().connect(
    sigc::mem_fun(*this, &SubtitleView::on_edited_text));

  column->set_resizable(true);
//  column->set_expand(true);

	m_columns["text"] = column;
}

/*
 *
 */
void SubtitleView::createColumnMarginR()
{
  Gtk::TreeViewColumn* column = NULL;
  Gtk::CellRendererText* renderer = NULL;
  
  column = manage(new Gtk::TreeViewColumn(_("R")));
  renderer = manage(new Gtk::CellRendererText);
  
  column->pack_start(*renderer, false);
  column->add_attribute(renderer->property_text(), m_column.marginR);
  
  renderer->property_editable() = true;

  renderer->signal_edited().connect(
    sigc::mem_fun(*this, &SubtitleView::on_edited_margin_r));

  append_column(*column);

	m_columns["marginR"] = column;
}

/*
 *
 */
void SubtitleView::createColumnMarginL()
{
  Gtk::TreeViewColumn* column = NULL;
  Gtk::CellRendererText* renderer = NULL;
  
  column = manage(new Gtk::TreeViewColumn(_("L")));
  renderer = manage(new Gtk::CellRendererText);
  
  column->pack_start(*renderer, false);
  column->add_attribute(renderer->property_text(), m_column.marginL);
  
  renderer->property_editable() = true;

  renderer->signal_edited().connect(
    sigc::mem_fun(*this, &SubtitleView::on_edited_margin_l));

  append_column(*column);

	m_columns["marginL"] = column;
}

/*
 *
 */
void SubtitleView::createColumnMarginV()
{
  Gtk::TreeViewColumn* column = NULL;
  Gtk::CellRendererText* renderer = NULL;
  
  column = manage(new Gtk::TreeViewColumn(_("V")));
  renderer = manage(new Gtk::CellRendererText);
  
  column->pack_start(*renderer, false);
  column->add_attribute(renderer->property_text(), m_column.marginV);
  
  renderer->property_editable() = true;

  renderer->signal_edited().connect(
    sigc::mem_fun(*this, &SubtitleView::on_edited_margin_v));

  append_column(*column);

	m_columns["marginV"] = column;
}

/*
 *  retourne l'item select ou NULL
 */
Gtk::TreeIter SubtitleView::getSelected()
{
  return get_selection()->get_selected();
}


/*
 * 
 */
void SubtitleView::on_edited_layer( const Glib::ustring &path, const Glib::ustring &value)
{
  Gtk::TreeIter iter = m_subtitleModel->get_iter(path);
  if(iter)
  {
		int val = 0;
		if(from_string(value, val))
			(*iter)[m_column.layer] = value;
  }
}


/*
 *  callback utiliser pour modifier le temps directement depuis la list (treeview)
 */
void SubtitleView::on_edited_start( const Glib::ustring &path, 
                                    const Glib::ustring &newtext)
{
  Gtk::TreeIter iter = m_subtitleModel->get_iter(path);
  if(iter)
  {
    int hours=0, mins=0, secs=0, msecs=0;
    
    int res = sscanf(newtext.c_str(), "%d:%d:%d,%d", &hours,&mins,&secs,&msecs);

    if(res == 4)
    {
      SubtitleTime time(hours,mins,secs,msecs);
      // ici on convertir en MSecs pour ensuite le reconvertir en temps normal (h,m,s,ms)
      // afin de pouvoir recevoir des chiffres "bizarre" ex: "00:05:99,000" (99 secs et faux)
      // il y aura donc une conversion ce qui va donner : "00:06:39,000"
			(*iter)[m_column.start] = time.str();
			
    }
    else
    {
      std::cerr << "on_edited_start::invalide> " << newtext << std::endl;
    }
  }
}

/*
 *  callback utiliser pour modifier le temps directement depuis la list (treeview)
 */
void SubtitleView::on_edited_end( const Glib::ustring &path, 
                                  const Glib::ustring &newtext)
{
  Gtk::TreeIter iter = m_subtitleModel->get_iter(path);
  if(iter)
  {
    int hours=0, mins=0, secs=0, msecs=0;
    
    int res = sscanf(newtext.c_str(), "%d:%d:%d,%d", &hours,&mins,&secs,&msecs);

    if(res == 4)
    {
      SubtitleTime time(hours,mins,secs,msecs);
      // ici on convertir en MSecs pour ensuite le reconvertir en temps normal (h,m,s,ms)
      // afin de pouvoir recevoir des chiffres "bizarre" ex: "00:05:99,000" (99 secs et faux)
      // il y aura donc une conversion ce qui va donner : "00:06:39,000"
			(*iter)[m_column.end] = time.str();
    }
    else
    {
      std::cerr << "on_edited_end::invalide> " << newtext << std::endl;
    }
  }
}

/*
 *  callback utiliser pour modifier le texte
 */
void SubtitleView::on_edited_text( const Glib::ustring &path, 
                                    const Glib::ustring &newtext)
{
  Gtk::TreeIter iter = m_subtitleModel->get_iter(path);
  if(iter)
  {
		(*iter)[m_column.text] = newtext;
  }
}

/*
 *  callback utiliser pour modifier le style a partir d'un menu
 */
void SubtitleView::on_edited_style( const Glib::ustring &path, 
                                    const Glib::ustring &newstyle)
{
  Gtk::TreeIter iter = m_subtitleModel->get_iter(path);
  if(iter)
  {
		(*iter)[m_column.style] = newstyle;
  }
}

/*
 *  callback utiliser pour modifier le nom
 */
void SubtitleView::on_edited_name( const Glib::ustring &path, 
                                    const Glib::ustring &newname)
{
  Gtk::TreeIter iter = m_subtitleModel->get_iter(path);
  if(iter)
  {
		(*iter)[m_column.name] = newname;
  }
}

/*
 * 
 */
void SubtitleView::on_edited_margin_l( const Glib::ustring &path, const Glib::ustring &value)
{
  Gtk::TreeIter iter = m_subtitleModel->get_iter(path);
  if(iter)
  {
		int val = 0;
		if(from_string(value, val))
			(*iter)[m_column.marginL] = value;
  }
}

/*
 * 
 */
void SubtitleView::on_edited_margin_r( const Glib::ustring &path, const Glib::ustring &value)
{
  Gtk::TreeIter iter = m_subtitleModel->get_iter(path);
  if(iter)
  {
		int val = 0;
		if(from_string(value, val))
			(*iter)[m_column.marginR] = value;
  }
}

/*
 * 
 */
void SubtitleView::on_edited_margin_v( const Glib::ustring &path, const Glib::ustring &value)
{
  Gtk::TreeIter iter = m_subtitleModel->get_iter(path);
  if(iter)
  {
		int val = 0;
		if(from_string(value, val))
			(*iter)[m_column.marginV] = value;
  }
}

/*
 *
 */
void SubtitleView::select_and_set_cursor(const Gtk::TreeIter &iter)
{
	get_selection()->select(iter);
  set_cursor(m_subtitleModel->get_path(iter), *m_columns["text"], false);
}


/*
 *	passe d'une vue simple pour le format SRT 
 *	a la vue total (SSA)
 *	afficher certaine colonne ou non
 *	"srt" ou "ssa"
 */
void SubtitleView::set_view_type(const Glib::ustring &type)
{
	if(type == "srt")
	{
		m_columns["num"]->set_visible(true);
		m_columns["layer"]->set_visible(false);
		m_columns["start"]->set_visible(true);
		m_columns["end"]->set_visible(true);
		m_columns["style"]->set_visible(false);
		m_columns["name"]->set_visible(false);
		m_columns["marginL"]->set_visible(false);
		m_columns["marginR"]->set_visible(false);
		m_columns["marginV"]->set_visible(false);
		m_columns["text"]->set_visible(true);
//	m_columns["effect"]->set_visible(false);
	}
	else if(type == "ssa")
	{
		m_columns["num"]->set_visible(true);
		m_columns["layer"]->set_visible(true);
		m_columns["start"]->set_visible(true);
		m_columns["end"]->set_visible(true);
		m_columns["style"]->set_visible(true);
		m_columns["name"]->set_visible(true);
		m_columns["marginL"]->set_visible(false);
		m_columns["marginR"]->set_visible(false);
		m_columns["marginV"]->set_visible(false);
		m_columns["text"]->set_visible(true);
//	m_columns["effect"]->set_visible(false);
	}
	else
		std::cerr << "SubtitleView::set_view_type type no found :" << type << std::endl;
}
