#include "utility.h"
#include <string>
#include <glibmm.h>
#include <iostream>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


static gchar* encodings [] = {
  "ISO-8859-1",
  "ISO-8859-2",
  "ISO-8859-3",
  "ISO-8859-4",
  "ISO-8859-5",
  "ISO-8859-6",
  "ISO-8859-7",
  "ISO-8859-8",
  "ISO-8859-8-I",
  "ISO-8859-9",
  "ISO-8859-10",
  "ISO-8859-13",
  "ISO-8859-14",
  "ISO-8859-15",
  "ISO-8859-16",
  
  "UTF-7",
  "UTF-16",
  "UCS-2",
  "UCS-4",
  
  "ARMSCII-8",
  "BIG5",
  "BIG5-HKSCS",
  "CP866",
  
  "EUC-JP",
  "EUC-KR",
  "EUC-TW",
  
  "GB18030",
  "GB2312",
  "GBK",
  "GEORGIAN-ACADEMY",
  "HZ",
  
  "IBM850",
  "IBM852",
  "IBM855",
  "IBM857",
  "IBM862",
  "IBM864",
  
  "ISO-2022-JP",
  "ISO-2022-KR",
  "ISO-IR-111",
  "JOHAB",
  "KOI8R",
  "KOI8-R",
  "KOI8U",
  
  "SHIFT_JIS",
  "TCVN",
  "TIS-620",
  "UHC",
  "VISCII",
  
  "WINDOWS-1250",
  "WINDOWS-1251",
  "WINDOWS-1252",
  "WINDOWS-1253",
  "WINDOWS-1254",
  "WINDOWS-1255",
  "WINDOWS-1256",
  "WINDOWS-1257",
  "WINDOWS-1258"
};


std::string last_charset="ISO-8859-15";

/*
 *
 */
bool check_end_char(const gchar c)
{
  switch(c)
  {
  case '\0':
  case '\t':
  case '\015':
  case '\n':
    return true;
  default:
   return false;
  }
  return false;
}


/*
 *
 */
Glib::ustring check_end_char(const Glib::ustring &str)
{
  Glib::ustring::size_type pos = str.find('\015');
  if(pos != Glib::ustring::npos)
    return str.substr(0, pos);
  return str;
}


/*
 *
 */
bool convert_to_utf8_from_charset( const std::string &content, 
                                   const std::string &charset,
                                   Glib::ustring &result)
{
  // check content & charset...
  try
  {
    Glib::ustring res = Glib::convert(content,"UTF-8", charset);
    if(res.validate())
    {
      result = res;
      return true;
    }
  }
  catch(Glib::ConvertError& ex)
  {
  }
  
  return false;
}


/*
 *
 */
Glib::ustring convert_to_utf8(const std::string &src)
{		
	// on teste si c'est de l'utf-8
  Glib::ustring res(src);
  if(res.validate())
  {
    return res;
  }

	if(convert_to_utf8_from_charset(src, last_charset, res))
    return res;
  else
  {
    // dans ce cas on teste tout les encodings possibles
    int size = sizeof(encodings) / sizeof(encodings[0]);
    for(int i=0; i<size; ++i)
    {
      if(convert_to_utf8_from_charset(src, encodings[i], res))
      {
				last_charset = encodings[i];
				return res;
      }
    }
  }
  
  return "";
}

/*
 *	verifie si la chaine est valide du type "00:00:00,000"
 */
bool check_time(const Glib::ustring str)
{
	int time[4];

	if(sscanf(str.c_str(), "%d:%d:%d,%d", &time[0], &time[1], &time[2], &time[3]) == 4)
		return true;
	return false;
}


/*
 *
 */
bool uint2color(unsigned int src, Gdk::Color &color_dest)
{
	unsigned int r = ((src & 0x0000FF) >> 0) * 257;
	unsigned int g = ((src & 0x00FF00) >> 8) * 257;
	unsigned int b = ((src & 0xFF0000) >> 16) * 257;

	color_dest.set_rgb(r,g,b);

	//std::cout << "uint2color: " << src << " : " << r << " - " << g << " - " << b << std::endl;
	return true;
}

bool color2uint(Gdk::Color &src, unsigned int &dest)
{
	dest = 0;

	unsigned int r = src.get_red() / 257;
	unsigned int g = src.get_green() / 257;
	unsigned int b = src.get_blue() / 257;


	dest = b << 16  | g  << 8 | r << 0;

	//std::cout << "color2uint: " << dest << " : " << r << " - " << g << " - " << b << std::endl;
	return true;
}

/*
 *	convertir str en n'importe quel type
 */
/*
template<class T>
bool from_string(const std::string &src, T& dest)
{
	std::istringstream s(src);
	// return s >> dest != 0;

	bool state = s >> dest != 0;

	g_return_val_if_fail(state, false);

	return true;
}
*/

/*
 *
 */
Glib::ustring get_share_dir(const Glib::ustring &file)
{
	Glib::ustring filename;
	
	filename = Glib::build_filename(PACKAGE_SHARE_DIR, file);

	if(Glib::file_test(filename, Glib::FILE_TEST_EXISTS))
		return filename;

	filename = Glib::build_filename("share", file);

	if(Glib::file_test(filename, Glib::FILE_TEST_EXISTS))
		return filename;
	
	std::cerr << "get_share_dir not found:" << file << std::endl;
	return "";
}
