#ifndef _utility_h
#define _utility_h

/*
 *  subtitle editor
 *  Copyright @ 2005-2006, IDJAAD Djamel (aka Kitone)
 *
 *  Contact: kitone at free point fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author IDJAAD Djamel (kitone at free point fr)
 */

#include <libintl.h>
#include <glibmm/ustring.h>
#include <gdkmm/color.h>
#include <sstream>
/*
 *
 */
#define _(srt) gettext(srt)

/*
 *
 */
Glib::ustring get_share_dir(const Glib::ustring &file);

/*
 *	verifie si la chaine est valide du type "00:00:00,000"
 */
bool check_time(const Glib::ustring str);

/*
 *
 */
Glib::ustring convert_to_utf8(const std::string &src);

/*
 *
 */
Glib::ustring check_end_char(const Glib::ustring &str);


/*
 *	convertir str en n'importe quel type
 */
template<class T>
bool from_string(const std::string &src, T& dest)
{
	std::istringstream s(src);
	// return s >> dest != 0;

	bool state = s >> dest != 0;

	g_return_val_if_fail(state, false);

	return true;
}

template<class T>
bool from_string(const Glib::ustring &src, T& dest)
{
	std::istringstream s(src);
	// return s >> dest != 0;

	bool state = s >> dest != 0;

	g_return_val_if_fail(state, false);

	return true;
}

/*
 *	convertir n'importe quoi en string
 */
template<class T>
std::string to_string(T &src)
{
	std::ostringstream oss;
	oss << src;
	
	return oss.str();
}


static gchar* character_encodings [] = {
  "ISO-8859-1",
  "ISO-8859-2",
  "ISO-8859-3",
  "ISO-8859-4",
  "ISO-8859-5",
  "ISO-8859-6",
  "ISO-8859-7",
  "ISO-8859-8",
  "ISO-8859-8-I",
  "ISO-8859-9",
  "ISO-8859-10",
  "ISO-8859-13",
  "ISO-8859-14",
  "ISO-8859-15",
  "ISO-8859-16",
  
  "UTF-7",
  "UTF-8",
  "UTF-16",
  "UCS-2",
  "UCS-4",
  
  "ARMSCII-8",
  "BIG5",
  "BIG5-HKSCS",
  "CP866",
  
  "EUC-JP",
  "EUC-KR",
  "EUC-TW",
  
  "GB18030",
  "GB2312",
  "GBK",
  "GEORGIAN-ACADEMY",
  "HZ",
  
  "IBM850",
  "IBM852",
  "IBM855",
  "IBM857",
  "IBM862",
  "IBM864",
  
  "ISO-2022-JP",
  "ISO-2022-KR",
  "ISO-IR-111",
  "JOHAB",
  "KOI8R",
  "KOI8-R",
  "KOI8U",
  
  "SHIFT_JIS",
  "TCVN",
  "TIS-620",
  "UHC",
  "VISCII",
  
  "WINDOWS-1250",
  "WINDOWS-1251",
  "WINDOWS-1252",
  "WINDOWS-1253",
  "WINDOWS-1254",
  "WINDOWS-1255",
  "WINDOWS-1256",
  "WINDOWS-1257",
  "WINDOWS-1258"
};

/*
 *	rgba >> 0, rgba >> 8, rgba >> 16
 */
bool uint2color(unsigned int rgba_src, Gdk::Color &color_dest);

bool color2uint(Gdk::Color &color_src, unsigned int &rgba_dest);

#endif//_utility_h
